import unittest
import os
import sys
import shutil
import copy
import zipfile
import json
from resilient_circuits.util.ext.ExtCreate import ExtCreate, PATH_DEFAULT_ICON_EXTENSION_LOGO, PATH_DEFAULT_ICON_COMPANY_LOGO
from resilient_circuits.util.ext import ExtException

# Import mock_data (need to add path to support relative imports in PY3)
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from ext_tests.mock_data.mock_data import MOCK_INTEGRATION_NAME, MOCK_INTEGRATION_URL, MOCK_INTEGRATION_LONG_DESCRIPTION, mock_import_definition, mock_import_definition_tagged, mock_config_str, mock_config_list, mock_setup_py_file_lines, mock_parsed_setup_py_attributes, mock_icon_extension_logo, mock_icon_company_logo, mock_message_destination_to_be_tagged, mock_message_destination_tagged, mock_extension_zip_file_structure, mock_executables_zip_file_structure

path_this_dir = os.path.dirname(os.path.realpath(__file__))
path_temp_test_dir = os.path.join(path_this_dir, "test_temp")
path_fn_mock_integration = os.path.join(path_this_dir, "mock_data", "fn_mock_integration")
path_mock_setup_py_file = os.path.join(path_fn_mock_integration, "setup.py")
path_mock_customize_py_file = os.path.join(path_fn_mock_integration, "fn_mock_integration", "util", "customize.py")
path_mock_config_py_file = os.path.join(path_fn_mock_integration, "fn_mock_integration", "util", "config.py")
path_mock_bd_true_tar = os.path.join(path_this_dir, "mock_data", "built_distributions", "true_tar_fn_mock_integration-1.0.0.tar.gz")
path_mock_bd_true_zip = os.path.join(path_this_dir, "mock_data", "built_distributions", "true_zip_fn_mock_integration-1.0.0.zip")
path_mock_bd_zipped_tar = os.path.join(path_this_dir, "mock_data", "built_distributions", "zipped_tar_fn_mock_integration-1.0.0.zip")

path_mock_export_res = os.path.join(path_this_dir, "mock_data", "ext-fn_mock_integration-1.0.0", "export.res")
path_mock_extension_json = os.path.join(path_this_dir, "mock_data", "ext-fn_mock_integration-1.0.0", "extension.json")
path_mock_exectuable_json = os.path.join(path_this_dir, "mock_data", "ext-fn_mock_integration-1.0.0", "executables", "executable.json")


def get_dict_from_json_file(file_path):
    dict_to_return = {}
    with open(file_path, 'r') as the_file:
            dict_to_return = json.load(the_file)
    return dict_to_return


class ExtCreateClassTestIndividualFns(unittest.TestCase):

    maxDiff = None

    def setUp(self):
        # assertRaisesRegexp renamed to assertRaisesRegex in PY3.2
        if sys.version_info < (3, 2):
            self.assertRaisesRegex = self.assertRaisesRegexp

        self.ext_create_class = ExtCreate("ext:package")
        self.original_import_definition = copy.deepcopy(mock_import_definition)
        self.original_mock_message_destination_to_be_tagged = copy.deepcopy(mock_message_destination_to_be_tagged[0])

    def tearDown(self):
        # Reset mock_import_definition object
        mock_import_definition.clear()
        mock_import_definition.update(copy.deepcopy(self.original_import_definition))

        # Reset mock_message_destination_to_be_tagged
        mock_message_destination_to_be_tagged[0].clear()
        mock_message_destination_to_be_tagged[0].update(copy.deepcopy(self.original_mock_message_destination_to_be_tagged))

    def test_get_import_definition_from_customize_py(self):
        import_definition = self.ext_create_class.__get_import_definition_from_customize_py__(path_mock_customize_py_file)

        self.assertEqual(import_definition, mock_import_definition)

    def test_get_configs_from_config_py(self):
        the_config_str, the_config_list = self.ext_create_class.__get_configs_from_config_py__(path_mock_config_py_file)

        self.assertEqual(the_config_str, mock_config_str)
        self.assertEqual(the_config_list, mock_config_list)

    def test_parse_setup_attribute(self):
        the_parsed_name_value = self.ext_create_class.__parse_setup_attribute__(path_mock_setup_py_file, mock_setup_py_file_lines, "name")
        self.assertEqual(the_parsed_name_value, MOCK_INTEGRATION_NAME)

        the_parsed_url_value = self.ext_create_class.__parse_setup_attribute__(path_mock_setup_py_file, mock_setup_py_file_lines, "url")
        self.assertEqual(the_parsed_url_value, MOCK_INTEGRATION_URL)

        the_parsed_long_description_value = self.ext_create_class.__parse_setup_attribute__(path_mock_setup_py_file, mock_setup_py_file_lines, "long_description")
        self.assertEqual(the_parsed_long_description_value, MOCK_INTEGRATION_LONG_DESCRIPTION)

    def test_parse_setup_py(self):
        attribute_names = self.ext_create_class.supported_setup_py_attribute_names
        parsed_setup_py_attributes = self.ext_create_class.__parse_setup_py__(path_mock_setup_py_file, attribute_names)
        self.assertEqual(parsed_setup_py_attributes, mock_parsed_setup_py_attributes)

    def test_get_icon(self):

        path_extension_logo = os.path.join(path_fn_mock_integration, "icons", "extension_logo.png")
        path_company_logo = os.path.join(path_fn_mock_integration, "icons", "company_logo.png")

        path_to_corrupt_jpg_icon = os.path.join(path_fn_mock_integration, "icons", "mock_corrupt_icon.jpg")
        path_to_corrupt_png_icon = os.path.join(path_fn_mock_integration, "icons", "mock_corrupt_icon.png")

        # Test getting extension_logo
        extension_logo_as_base64 = self.ext_create_class.__get_icon__(os.path.basename(PATH_DEFAULT_ICON_EXTENSION_LOGO), path_extension_logo, 200, 72, PATH_DEFAULT_ICON_EXTENSION_LOGO)
        self.assertEqual(extension_logo_as_base64, mock_icon_extension_logo)

        # Test getting default extension_logo
        extension_logo_as_base64 = self.ext_create_class.__get_icon__(os.path.basename(PATH_DEFAULT_ICON_EXTENSION_LOGO), "", 200, 72, PATH_DEFAULT_ICON_EXTENSION_LOGO)
        self.assertEqual(extension_logo_as_base64, mock_icon_extension_logo)

        # Test getting company_logo
        company_logo_as_base64 = self.ext_create_class.__get_icon__(os.path.basename(PATH_DEFAULT_ICON_COMPANY_LOGO), path_company_logo, 100, 100, PATH_DEFAULT_ICON_COMPANY_LOGO)
        self.assertEqual(company_logo_as_base64, mock_icon_company_logo)

        # Test getting default company_logo
        company_logo_as_base64 = self.ext_create_class.__get_icon__(os.path.basename(PATH_DEFAULT_ICON_COMPANY_LOGO), "", 100, 100, PATH_DEFAULT_ICON_COMPANY_LOGO)
        self.assertEqual(company_logo_as_base64, mock_icon_company_logo)

        # Test invalid paths
        with self.assertRaisesRegex(OSError, "Could not find valid icon file. Looked at two locations:"):
            self.ext_create_class.__get_icon__(os.path.basename(PATH_DEFAULT_ICON_COMPANY_LOGO), "", 200, 72, "")

        # Test not .png
        with self.assertRaisesRegex(ExtException, ".jpg is not a supported icon file type. Icon file must be .png"):
            self.ext_create_class.__get_icon__(os.path.basename(PATH_DEFAULT_ICON_EXTENSION_LOGO), path_to_corrupt_jpg_icon, 10, 10, PATH_DEFAULT_ICON_EXTENSION_LOGO)

        # Test corrupt .png
        with self.assertRaisesRegex(ExtException, "Icon file corrupt"):
            self.ext_create_class.__get_icon__(os.path.basename(PATH_DEFAULT_ICON_EXTENSION_LOGO), path_to_corrupt_png_icon, 10, 10, PATH_DEFAULT_ICON_EXTENSION_LOGO)

        # Test invalid resolution
        with self.assertRaisesRegex(ExtException, "Resolution must be 10x10"):
            self.ext_create_class.__get_icon__(os.path.basename(PATH_DEFAULT_ICON_EXTENSION_LOGO), path_extension_logo, 10, 10, PATH_DEFAULT_ICON_EXTENSION_LOGO)

    def test_add_tag(self):
        tag_name = MOCK_INTEGRATION_NAME

        # Test adding the tag
        tagged_md = self.ext_create_class.__add_tag__(tag_name, mock_message_destination_to_be_tagged)
        self.assertEqual(tagged_md, mock_message_destination_tagged)

        # Test invalid list_of_objs
        with self.assertRaisesRegex(ExtException, "is not a List"):
            self.ext_create_class.__add_tag__(tag_name, "")

    def test_add_tag_to_import_definition(self):
        tag_name = MOCK_INTEGRATION_NAME
        supported_res_obj_names = self.ext_create_class.supported_res_obj_names
        tagged_import_definition = self.ext_create_class.__add_tag_to_import_definition__(tag_name, supported_res_obj_names, mock_import_definition)

        self.assertEqual(tagged_import_definition, mock_import_definition_tagged)


class ExtCreateClassTestCreateExtension(unittest.TestCase):

    maxDiff = None

    def setUp(self):
        self.ext_create_class = ExtCreate("ext:package")

        # Create temp dir
        os.makedirs(path_temp_test_dir)

    def tearDown(self):
        # Remove temp dir
        shutil.rmtree(path_temp_test_dir)

    def create_the_extension(self, path_built_distribution):
        return self.ext_create_class.create_extension(
            path_setup_py_file=path_mock_setup_py_file,
            path_customize_py_file=path_mock_customize_py_file,
            path_config_py_file=path_mock_config_py_file,
            output_dir=path_temp_test_dir,
            path_built_distribution=path_built_distribution
        )

    def validate_creation(self, path_the_extension_zip):
        path_created_extension_zip = os.path.join(path_temp_test_dir, "ext-fn_mock_integration-1.0.0.zip")

        self.assertEqual(path_the_extension_zip, path_created_extension_zip)
        self.assertTrue(zipfile.is_zipfile(path_the_extension_zip))

    def validate_extension_zip_folder_structure(self, path_the_extension_zip):
        # Open the zip
        with zipfile.ZipFile(file=path_the_extension_zip, mode="r") as zip_file:
            # Get a List of all the member names
            zip_file_structure = zip_file.namelist()

        self.assertEqual(zip_file_structure, mock_extension_zip_file_structure)

    def validate_executables_folder_structure(self, path_the_extension_zip):
        # Open the extension_zip
        with zipfile.ZipFile(file=path_the_extension_zip, mode="r") as the_extension_zip_file:
            # Extract the executables_zip
            path_the_executable_zip_file = the_extension_zip_file.extract(member="executables/exe-{0}-1.0.0.zip".format(MOCK_INTEGRATION_NAME), path=path_temp_test_dir)

        # Open the executables_zip
        with zipfile.ZipFile(file=path_the_executable_zip_file, mode="r") as the_executable_zip_file:
            the_executable_zip_file_structure = the_executable_zip_file.namelist()

        self.assertEqual(the_executable_zip_file_structure, mock_executables_zip_file_structure)

    def validate_export_res_and_extension_json(self, path_the_extension_zip):
        # Get the mock data
        mock_export_res = get_dict_from_json_file(path_mock_export_res)
        mock_extension_json = get_dict_from_json_file(path_mock_extension_json)

        # Extract the zip
        with zipfile.ZipFile(file=path_the_extension_zip, mode="r") as zip_file:
            zip_file.extractall(path=path_temp_test_dir)

        # Get the export_res and extension_json
        export_res = get_dict_from_json_file(os.path.join(path_temp_test_dir, "export.res"))
        extension_json = get_dict_from_json_file(os.path.join(path_temp_test_dir, "extension.json"))

        # Compare
        self.assertEqual(export_res, mock_export_res)
        self.assertEqual(mock_extension_json, extension_json)

    def validate_executable_json(self, path_the_extension_zip):
        # Get the mock data
        mock_executable_json = get_dict_from_json_file(path_mock_exectuable_json)

        # Open the extension_zip
        with zipfile.ZipFile(file=path_the_extension_zip, mode="r") as the_extension_zip_file:
            # Extract the executables_zip
            path_the_executable_zip_file = the_extension_zip_file.extract(member="executables/exe-{0}-1.0.0.zip".format(MOCK_INTEGRATION_NAME), path=path_temp_test_dir)

        # Open the executables_zip
        with zipfile.ZipFile(file=path_the_executable_zip_file, mode="r") as the_executable_zip_file:
            the_executable_zip_file.extractall(path=path_temp_test_dir)

        # Get the executable.json
        executable_json = get_dict_from_json_file(os.path.join(path_temp_test_dir, "executable.json"))

        # Compare
        self.assertEqual(executable_json, mock_executable_json)

    #################
    # Test TRUE TAR #
    #################
    def test_true_tar_creation(self):
        # Create the extension
        path_the_extension_zip = self.create_the_extension(path_mock_bd_true_tar)

        # Validate creation
        self.validate_creation(path_the_extension_zip)

    def test_true_tar_valid_extension_zip_folder_structure(self):
        # Create the extension
        path_the_extension_zip = self.create_the_extension(path_mock_bd_true_tar)

        # Validate folder structure
        self.validate_extension_zip_folder_structure(path_the_extension_zip)

    def test_true_tar_valid_executables_folder_structure(self):
        # Create the extension
        path_the_extension_zip = self.create_the_extension(path_mock_bd_true_tar)

        # Validate folder structure
        self.validate_executables_folder_structure(path_the_extension_zip)

    def test_true_tar_valid_export_res_and_extension_json(self):
        # Create the extension
        path_the_extension_zip = self.create_the_extension(path_mock_bd_true_tar)

        # Validate export.res and extension.json
        self.validate_export_res_and_extension_json(path_the_extension_zip)

    def test_true_tar_valid_executables_json(self):
        # Create the extension
        path_the_extension_zip = self.create_the_extension(path_mock_bd_true_tar)

        # Validate export.res and extension.json
        self.validate_executable_json(path_the_extension_zip)

    #################
    # Test TRUE ZIP #
    #################
    def test_true_zip_creation(self):
        # Create the extension
        path_the_extension_zip = self.create_the_extension(path_mock_bd_true_zip)

        # Validate creation
        self.validate_creation(path_the_extension_zip)

    def test_true_zip_valid_extension_zip_folder_structure(self):
        # Create the extension
        path_the_extension_zip = self.create_the_extension(path_mock_bd_true_zip)

        # Validate folder structure
        self.validate_extension_zip_folder_structure(path_the_extension_zip)

    def test_true_zip_valid_executables_folder_structure(self):
        # Create the extension
        path_the_extension_zip = self.create_the_extension(path_mock_bd_true_zip)

        # Validate folder structure
        self.validate_executables_folder_structure(path_the_extension_zip)

    def test_true_zip_valid_export_res_and_extension_json(self):
        # Create the extension
        path_the_extension_zip = self.create_the_extension(path_mock_bd_true_zip)

        # Validate export.res and extension.json
        self.validate_export_res_and_extension_json(path_the_extension_zip)

    def test_true_zip_valid_executables_json(self):
        # Create the extension
        path_the_extension_zip = self.create_the_extension(path_mock_bd_true_zip)

        # Validate export.res and extension.json
        self.validate_executable_json(path_the_extension_zip)

    ###################
    # Test ZIPPED TAR #
    ###################
    def test_zipped_tar_creation(self):
        # Create the extension
        path_the_extension_zip = self.create_the_extension(path_mock_bd_zipped_tar)

        # Validate creation
        self.validate_creation(path_the_extension_zip)

    def test_zipped_tar_valid_extension_zip_folder_structure(self):
        # Create the extension
        path_the_extension_zip = self.create_the_extension(path_mock_bd_zipped_tar)

        # Validate folder structure
        self.validate_extension_zip_folder_structure(path_the_extension_zip)

    def test_zipped_tar_valid_executables_folder_structure(self):
        # Create the extension
        path_the_extension_zip = self.create_the_extension(path_mock_bd_zipped_tar)

        # Validate folder structure
        self.validate_executables_folder_structure(path_the_extension_zip)

    def test_zipped_tar_valid_export_res_and_extension_json(self):
        # Create the extension
        path_the_extension_zip = self.create_the_extension(path_mock_bd_zipped_tar)

        # Validate export.res and extension.json
        self.validate_export_res_and_extension_json(path_the_extension_zip)

    def test_zipped_tar_valid_executables_json(self):
        # Create the extension
        path_the_extension_zip = self.create_the_extension(path_mock_bd_zipped_tar)

        # Validate export.res and extension.json
        self.validate_executable_json(path_the_extension_zip)


import unittest
import os
import sys
import shutil
import stat
from resilient_circuits.util.ext.Ext import Ext
from resilient_circuits.util.ext import ExtException


class ExtClassTestIndividualFns(unittest.TestCase):

    def setUp(self):
        # assertRaisesRegexp renamed to assertRaisesRegex in PY3.2
        if sys.version_info < (3, 2):
            self.assertRaisesRegex = self.assertRaisesRegexp

        self.ext_class = Ext("ext:package")
        self.dir_path = os.path.dirname(os.path.realpath(__file__))
        self.temp_dir = os.path.join(self.dir_path, "test_temp")
        self.mock_file_contents = "Write this to mock file"
        os.makedirs(self.temp_dir)

    def tearDown(self):
        shutil.rmtree(self.temp_dir)

    def test_read_write_file(self):

        temp_file = os.path.join(self.temp_dir, "mock_file.txt")

        self.ext_class.__write_file__(temp_file, self.mock_file_contents)
        self.assertTrue(os.path.isfile(temp_file))

        file_lines = self.ext_class.__read_file__(temp_file)
        self.assertIn(self.mock_file_contents, file_lines)

    def test_has_permissions(self):
        temp_permissions_file = os.path.join(self.temp_dir, "mock_permissions.txt")
        self.ext_class.__write_file__(temp_permissions_file, self.mock_file_contents)

        # Set permissions to Read only
        os.chmod(temp_permissions_file, stat.S_IRUSR)

        with self.assertRaisesRegex(ExtException, "User does not have WRITE permissions"):
            self.ext_class.__has_permissions__(os.W_OK, temp_permissions_file)

        # Set permissions to Write only
        os.chmod(temp_permissions_file, stat.S_IWUSR)

        with self.assertRaisesRegex(ExtException, "User does not have READ permissions"):
            self.ext_class.__has_permissions__(os.R_OK, temp_permissions_file)

    def test_validate_directory(self):

        non_exist_path = "/non_exits/path"

        with self.assertRaisesRegex(ExtException, "The path does not exist: " + non_exist_path):
            self.ext_class.__validate_directory__(os.R_OK, non_exist_path)

        exists_dir = os.path.join(self.temp_dir, "mock_existing_dir")
        os.makedirs(exists_dir)

        self.ext_class.__validate_directory__(os.R_OK, exists_dir)

    def test_validate_file_paths(self):

        non_exist_path = "/non_exits/path"
        with self.assertRaisesRegex(ExtException, "Could not find file: " + non_exist_path):
            self.ext_class.__validate_file_paths__(None, non_exist_path)

        exists_file = os.path.join(self.temp_dir, "mock_existing_file.txt")
        self.ext_class.__write_file__(exists_file, self.mock_file_contents)

        self.ext_class.__validate_file_paths__(None, exists_file)

    def test_is_valid_url(self):
        self.assertTrue(self.ext_class.__is_valid_url__("www.example.com"))
        self.assertTrue(self.ext_class.__is_valid_url__("example.com"))
        self.assertTrue(self.ext_class.__is_valid_url__("http://www.example.com"))
        self.assertTrue(self.ext_class.__is_valid_url__("https://www.example.com"))

        self.assertFalse(self.ext_class.__is_valid_url__(None))
        self.assertFalse(self.ext_class.__is_valid_url__("not a url"))
        self.assertFalse(self.ext_class.__is_valid_url__("https://www. example.com"))

    def test_is_valid_package_name(self):
        self.assertTrue(self.ext_class.__is_valid_package_name__("fn_valid_name"))
        self.assertTrue(self.ext_class.__is_valid_package_name__("fn_VALID"))
        self.assertTrue(self.ext_class.__is_valid_package_name__("fn"))
        self.assertTrue(self.ext_class.__is_valid_package_name__("fn_valid_name_123"))

        self.assertFalse(self.ext_class.__is_valid_package_name__("fn-not-valid"))
        self.assertFalse(self.ext_class.__is_valid_package_name__("fn -not-valid"))
        self.assertFalse(self.ext_class.__is_valid_package_name__("fn_!@#_not_valid"))

    def test_generate_uuid_from_string(self):
        the_string, the_uuid = "fn_test_package", "7627eab9-8500-cf1d-380d-14a2c4364acf"

        the_generated_uuid = self.ext_class.__generate_uuid_from_string__(the_string)

        self.assertEqual(the_generated_uuid, the_uuid)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2018. All Rights Reserved.

""" Command line tool to manage and run resilient-circuits """
from __future__ import absolute_import

import argparse
import logging
import os
import os.path
import sys
from collections import defaultdict
import pkg_resources
import resilient
import datetime
import time
import uuid
from resilient import ensure_unicode
from resilient_circuits.app import AppArgumentParser
from resilient_circuits.util.resilient_codegen import codegen_functions, codegen_package, codegen_reload_package, print_codegen_reload_commandline, extract_to_res
from resilient_circuits.util.resilient_customize import customize_resilient
from resilient_circuits.util.resilient_ext import ext_command_handler


if sys.version_info.major == 2:
    from io import open
else:
    unicode = str
try:
    # For all python < 3.2
    import backports.configparser as configparser
except ImportError:
    import configparser

try:
    from builtins import input
except ImportError:
    # Python 2
    from __builtin__ import raw_input as input

LOG = logging.getLogger("resilient_circuits_cmd_logger")
LOG.setLevel(logging.INFO)
LOG.addHandler(logging.StreamHandler())


def windows_service(service_args, res_circuits_args):
    """Register a windows service"""
    try:
        import win32serviceutil
        from resilient_circuits.bin import service_wrapper

        if res_circuits_args:
            # Set the command line arguments to pass to "resilient-circuits.exe run"
            service_wrapper.irms_svc.setResilientArgs(res_circuits_args)

        sys.argv = sys.argv[0:1] + service_args
        win32serviceutil.HandleCommandLine(service_wrapper.irms_svc)
    except ImportError:
        LOG.error("Requires PYWIN32 Package. Please download and install from: "
                  "https://sourceforge.net/projects/pywin32/files/pywin32/")


def supervisor_service():
    """Register a unix service with supervisord [deprecated]"""
    pass


def manage_service(service_args, res_circuits_args):
    """Register a windows or unix service"""
    if os.name == 'nt':
        windows_service(service_args, res_circuits_args)
    else:
        LOG.error("Not implemented")
        # supervisor_service(service_args, res_circuits_args)


def run(resilient_circuits_args, restartable=False, config_file=None):
    """Run resilient-circuits"""
    # Leave only the arguments for the run command
    if restartable:
        # import here b/c it is slow
        from resilient_circuits import app_restartable as app
    else:
        from resilient_circuits import app
    sys.argv = sys.argv[0:1] + resilient_circuits_args
    kwargs = {}
    if config_file:
        kwargs = {"config_file": config_file}
    app.run(**kwargs)


def list_installed(args):
    """print list of installed packages with their components"""
    LOG.debug("resilient-circuits.list")
    components = defaultdict(list)
    # Resilient packages can have several entry-points,
    # - resilient.circuits.config: produces default configuration sections
    # - resilient.circuits.customize: produces schema customizations
    # - resilient.circuits.components: identifies components that implement actions, functions, etc.
    # We want to list all components, but include (as "empty") packages that define the other entry-points too.
    entry_points = []
    entry_points.extend([ep for ep in pkg_resources.iter_entry_points('resilient.circuits.config')])
    entry_points.extend([ep for ep in pkg_resources.iter_entry_points('resilient.circuits.customize')])
    component_entry_points = [ep for ep in pkg_resources.iter_entry_points('resilient.circuits.components')]
    entry_points.extend(component_entry_points)
    LOG.debug(u"Found %d installed entry-points", len(entry_points))
    for ep in entry_points:
        components[ep.dist].append(ep)
    if not components:
        LOG.info(u"No resilient-circuits components are installed")
        return
    LOG.info(u"The following packages and components are installed:")
    for dist, component_list in components.items():
        if args.verbose:
            clist = "\n\t".join([str(ep) for ep in component_list if ep in component_entry_points])
            if clist == "":
                LOG.info(u"%s (%s):\n\t(Package does not define any components)",
                         dist.as_requirement(),
                         dist.egg_info)
            else:
                LOG.info(u"%s (%s):\n\t%s",
                         dist.as_requirement(),
                         dist.egg_info,
                         clist)
        else:
            clist = "\n\t".join([ep.name for ep in component_list if ep in component_entry_points])
            if clist == "":
                LOG.info(u"%s:\n\t(Package does not define any components)",
                         dist.as_requirement())
            else:
                LOG.info(u"%s:\n\t%s",
                         dist.as_requirement(),
                         clist)


def generate_default(install_list):
    """ return string containing entire default app.config """
    base_config_fn = pkg_resources.resource_filename("resilient_circuits", "data/app.config.base")
    entry_points = pkg_resources.iter_entry_points('resilient.circuits.configsection')
    additional_sections = []
    remaining_list = install_list[:] if install_list else []
    for entry in entry_points:
        dist = entry.dist
        package_name = entry.dist.project_name

        # if a list is provided, use it to filter which packages to add to the app.config file
        if install_list is not None and package_name not in remaining_list:
            LOG.debug("{} bypassed".format(package_name))
            continue
        elif package_name in remaining_list:
            remaining_list.remove(package_name)

        try:
            func = entry.load()
        except ImportError:
            LOG.exception(u"Failed to load configuration defaults for package '%s'", repr(dist))
            continue

        new_section = None
        try:
            config_data = func()
            if config_data:
                required_config = configparser.ConfigParser(interpolation=None)
                LOG.debug(u"Config Data String:\n%s", config_data)
                required_config.read_string(unicode(config_data))
                new_section = required_config.sections()[0]
        except:
            LOG.exception(u"Failed to get configuration defaults for package '%s'", repr(dist))
            continue

        if new_section:
            additional_sections.append(config_data)

    LOG.debug("Found %d sections to generate", len(additional_sections))

    if install_list and len(remaining_list) > 0:
        LOG.warning("%s not found. Check package name(s)", remaining_list)

    with open(base_config_fn, 'r') as base_config_file:
        base_config = base_config_file.read()
        return "\n\n".join(([base_config, ] + additional_sections))


def generate_or_update_config(args):
    """ Create or update config file based on installed components """
    usage_type = "CREATING" if args.create else "UPDATING"

    if not args.filename:
        # Use default config file in ~/.resilient/app.config and create directory if missing
        config_filename = os.path.expanduser(os.path.join("~", ".resilient", "app.config"))
        resilient_dir = os.path.dirname(config_filename)
        if not os.path.exists(resilient_dir):
            LOG.info(u"Creating %s", resilient_dir)
            os.makedirs(resilient_dir)
    else:
        config_filename = os.path.expandvars(os.path.expanduser(args.filename))
    file_exists = os.path.exists(config_filename)

    if file_exists:
        if args.create:
            choice = ""
            while choice not in ('y', 'n'):
                choice = input(u"%s exists. Do you want to overwrite? (y/n): " % config_filename)
            if choice == 'n':
                LOG.error(u"Config file creation cancelled.")
                return
    elif args.update:
        LOG.error(u"File %s does not exist. Update cancelled.", config_filename)
        return

    LOG.info(u"%s config file %s", usage_type, config_filename)

    if args.create:
        # Write out default file
        with open(config_filename, "w+", encoding="utf-8") as config_file:
            config_file.write(generate_default(args.install_list))
            LOG.info(u"Configuration file generated: %s", config_filename)
            LOG.info(u"Please manually edit with your specific configuration values.")

    else:
        # Update existing file
        config = configparser.ConfigParser(interpolation=None)
        updated = False
        with open(config_filename, "r", encoding="utf-8") as config_file:
            first_byte = config_file.read(1)
            if first_byte != u'\ufeff':
                # Not a BOM, no need to skip first byte
                config_file.seek(0)
            config.read_file(config_file)
            existing_sections = config.sections()

        entry_points = pkg_resources.iter_entry_points('resilient.circuits.configsection')
        remaining_list = args.install_list[:] if args.install_list else []

        with open(config_filename, "a", encoding="utf-8") as config_file:
            for entry in entry_points:
                dist = entry.dist
                package_name = entry.dist.project_name
                try:
                    func = entry.load()
                except ImportError:
                    LOG.exception(u"Failed to load configuration defaults for package '%s'", repr(dist))
                    continue

                new_section = None
                try:
                    config_data = func()
                    if config_data:
                        required_config = configparser.ConfigParser(interpolation=None)
                        LOG.debug(u"Config Data String:\n%s", config_data)
                        required_config.read_string(unicode(config_data))
                        new_section = required_config.sections()[0]
                except:
                    LOG.exception(u"Failed to get configuration defaults for package '%s'", repr(dist))
                    continue

                LOG.debug(u"Required Section: %s", new_section)
                if new_section and new_section not in existing_sections:
                    if args.install_list is None or package_name in remaining_list:
                        # Add the default data for this required section to the config file
                        LOG.info(u"Adding new section '%s' for '%s'", new_section, dist)
                        if package_name in remaining_list:
                            remaining_list.remove(package_name)

                        config_file.write(u"\n" + config_data)
                        updated = True
                else:
                    LOG.debug(u"Section '%s' already present, not adding", new_section)
                    LOG.debug(u"%s %s", new_section, package_name)
                    if package_name in remaining_list:
                        remaining_list.remove(package_name)

            if args.install_list and len(remaining_list) > 0:
                LOG.warning("%s not found. Check package name(s)", remaining_list)

            if updated:
                LOG.info(u"Update finished.  "
                         u"New sections may require manual edits with your specific configuration values.")
            else:
                LOG.info(u"No updates.")


def generate_code(args):
    """generate template code components from functions"""
    parser = AppArgumentParser(config_file=resilient.get_config_file())
    (opts, extra) = parser.parse_known_args()
    client = resilient.get_client(opts)

    if args.cmd == "extract" and args.output:
        extract_to_res(client, args.exportfile,
                          args.messagedestination, args.function, args.workflow, args.rule,
                          args.field, args.datatable, args.task, args.script, args.artifacttype,
                          args.output, args.zip)
    elif args.reload:
        codegen_reload_package(client, args)
    elif args.package:
        # codegen an installable package
        output_base = os.path.join(os.curdir, args.package)
        codegen_package(client, args.exportfile, args.package,
                        args.messagedestination, args.function, args.workflow, args.rule,
                        args.field, args.datatable, args.task, args.script, args.artifacttype,
                        os.path.expanduser(output_base))
    elif args.function:
        # codegen a component for one or more functions
        if len(args.function) > 1:
            default_name = "functions.py"
        else:
            default_name = "{}.py".format(args.function[0])
        output_dir = os.path.expanduser(opts["componentsdir"] or os.curdir)
        output_file = args.output or default_name
        if not output_file.endswith(".py"):
            output_file = output_file + ".py"
        codegen_functions(client, args.exportfile, args.function, args.workflow, args.rule, args.artifacttype,
                          output_dir, output_file)

def selftest(args):
    """loop through every selftest for every eligible package, call and store returned state,
        print out package and their selftest states"""

    components = defaultdict(list)

    # custom entry_point only for selftest functions
    selftest_entry_points = [ep for ep in pkg_resources.iter_entry_points('resilient.circuits.selftest')]
    for ep in selftest_entry_points:
        components[ep.dist].append(ep)

    if len(selftest_entry_points) == 0:
        LOG.info("No selftest entry points found.")
        return None

    # Generate opts array necessary for ResilientComponent instantiation
    opts = AppArgumentParser(config_file=resilient.get_config_file()).parse_args("", None);

    # make a copy
    install_list = list(args.install_list) if args.install_list else []

    for dist, component_list in components.items():
        if args.install_list is None or dist.project_name in install_list:
            # remove name from list
            if dist.project_name in install_list:
                install_list.remove(dist.project_name)

            # add an entry for the package
            LOG.info("%s: ", dist.project_name)
            for ep in component_list:
                # load the entry point
                f_selftest = ep.load()

                try:
                    # f_selftest is the selftest function, we pass the selftest resilient options in case it wants to use it
                    start_time_milliseconds = int(round(time.time() * 1000))

                    status = f_selftest(opts)

                    end_time_milliseconds = int(round(time.time() * 1000))

                    delta_milliseconds = end_time_milliseconds - start_time_milliseconds
                    delta_seconds = delta_milliseconds / 1000

                    if status["state"] is not None:
                       LOG.info("\t%s: %s, Elapsed time: %f seconds", ep.name, status["state"], delta_seconds)
                except Exception as e:
                    LOG.error("Error while calling %s. Exception: %s", ep.name, str(e))
                    continue

    # any missed packages?
    if len(install_list):
        LOG.warning("%s not found. Check package name(s)", install_list)


def find_workflow_by_programmatic_name(workflows, pname):
    for workflow in workflows:
        if workflow.get("programmatic_name") == pname:
            return workflow

    return None


def clone(args):
    parser = AppArgumentParser(config_file=resilient.get_config_file())
    (opts, extra) = parser.parse_known_args()
    client = resilient.get_client(opts)

    export_uri = "/configurations/exports/history"
    export_list = client.get(export_uri)["histories"]
    last_date = 0
    last_id = 0
    for export in export_list:
        if export["options"]["actions"] and export["options"]["phases_and_tasks"]:
            if export["date"] > last_date:
                last_date = export["date"]
                last_id = export["id"]
    if last_date == 0:
        LOG.error(u"ERROR: No suitable export is available.  "
                  u"Create an export for code generation. (Administrator Settings -> Organization -> Export).")
        return

    dt = datetime.datetime.utcfromtimestamp(last_date / 1000.0)
    LOG.info(u"Codegen is based on the organization export from {}.".format(dt))
    export_uri = "/configurations/exports/{}".format(last_id)
    export_data = client.get(export_uri)  # Get latest export

    new_export_data = export_data.copy()
    whitelist_dict_keys = ["incident_types", "fields"]  # Mandatory keys
    for dict_key in new_export_data:
        if dict_key not in whitelist_dict_keys and type(new_export_data[dict_key]) is list:
            new_export_data[dict_key] = []  # clear the new export data, the stuff we clear isn't necessary for cloning

    workflow_names = args.workflow  # names of workflow a (target) and b (new workflow)
    if workflow_names:  # if we're importing workflows
        if len(workflow_names) != 2:
            raise Exception("Only specify the original workflow api name and a new workflow api name")

        # Check that 'workflows' are available (v28 onward)
        workflow_defs = export_data.get("workflows")
        if workflow_defs is None:
            raise Exception("Export does not contain workflows")

        original_workflow_api_name = workflow_names[0]
        new_workflow_api_name = workflow_names[1]

        duplicate_check = find_workflow_by_programmatic_name(workflow_defs, new_workflow_api_name)
        if duplicate_check is not None:
            raise Exception("Workflow with the api name {} already exists".format(new_workflow_api_name))

        original_workflow = find_workflow_by_programmatic_name(workflow_defs, original_workflow_api_name)
        if original_workflow is None:
            raise Exception("Could not find original workflow {}".format(original_workflow_api_name))

        # This section just fills out the stuff we need to replace to duplicate
        new_workflow = original_workflow.copy()
        # Random UUID, not guaranteed to not collide but is extremely extremely extremely unlikely to collide
        new_workflow["uuid"] = str(uuid.uuid4())
        new_workflow["programmatic_name"] = new_workflow_api_name
        new_workflow["export_key"] = new_workflow_api_name
        old_workflow_name = new_workflow["name"]
        new_workflow["name"] = new_workflow_api_name
        new_workflow["content"]["workflow_id"] = new_workflow_api_name
        new_workflow["content"]["xml"] = new_workflow["content"]["xml"].replace(original_workflow_api_name,
                                                                                new_workflow_api_name)
        new_workflow["content"]["xml"] = new_workflow["content"]["xml"].replace(old_workflow_name,
                                                                                new_workflow_api_name)

        new_export_data["workflows"] = [new_workflow]

    uri = "/configurations/imports"
    result = client.post(uri, new_export_data)
    import_id = result["id"]  # if this isn't here and the response code is 200 OK, something went really wrong

    if result["status"] == "PENDING":
        result["status"] = "ACCEPTED"      # Have to confirm changes
        uri = "/configurations/imports/{}".format(import_id)
        client.put(uri, result)
        LOG.info("Imported successfully")
    else:
        raise Exception("Could not import because the server did not return an import ID")

def add_ext_arguments(cmd, ext_parser):
    """Add arguments to the given ext: command"""

    # Add cmd specific arguments
    if cmd == "ext:package":
        ext_parser.add_argument("-p",
            help="Path to the directory containing the setup.py file",
            default=os.getcwd(),
            required=True,
            metavar="path")

        ext_parser.add_argument("--keep-build-dir",
            help="Do not delete the dist/build directory",
            action="store_true")

    elif cmd == "ext:convert":
        ext_parser.add_argument("-p",
            help="Path to the (old) Integration that can be in .tar.gz or .zip format",
            required=True,
            metavar="path")

    # Add common (optional) arguments
    if cmd in ("ext:package", "ext:convert"):

        ext_parser.add_argument("--display-name",
            help="The Display Name to give the Extension",
            nargs="?",
            metavar="name")

    return ext_parser

def main():
    """Main commandline"""
    # create base parser for extract and codgen
    common_parser = argparse.ArgumentParser(add_help=False)

    # Options for 'codegen'
    common_parser.add_argument("-f", "--function",
                                type=ensure_unicode,
                                help="Generate code for the specified function(s)",
                                nargs="*")
    common_parser.add_argument("-m", "--messagedestination",
                                type=ensure_unicode,
                                help="Generate code for all functions that use the specified message destination(s)",
                                nargs="*")
    common_parser.add_argument("--workflow",
                                type=ensure_unicode,
                                help="Include customization data for workflow(s)",
                                nargs="*")
    common_parser.add_argument("--rule",
                                type=ensure_unicode,
                                help="Include customization data for rule(s)",
                                nargs="*")
    common_parser.add_argument("--field",
                                type=ensure_unicode,
                                help="Include customization data for incident field(s)",
                                nargs="*")
    common_parser.add_argument("--datatable",
                                type=ensure_unicode,
                                help="Include customization data for datatable(s)",
                                nargs="*")
    common_parser.add_argument("--task",
                                type=ensure_unicode,
                                help="Include customization data for automatic task(s)",
                                nargs="*")
    common_parser.add_argument("--script",
                                type=ensure_unicode,
                                help="Include customization data for script(s)",
                                nargs="*")
    common_parser.add_argument("--artifacttype",
                               type=ensure_unicode,
                               help="Include customization data for artifact types(s)",
                               nargs="*")
    common_parser.add_argument("--exportfile",
                                type=ensure_unicode,
                                help="Generate based on organization export file (.res)")
    common_parser.add_argument("-o", "--output",
                                type=ensure_unicode,
                                help="Output file name")


    parser = argparse.ArgumentParser()
    parser.add_argument("-v", "--verbose", help="Print debug output", action="store_true")

    subparsers = parser.add_subparsers(title="subcommands",
                                       help="one of these options must be provided",
                                       description="valid subcommands",
                                       dest="cmd")
    subparsers.required = True

    run_parser = subparsers.add_parser("run",
                                       help="Run the Resilient Circuits application")
    list_parser = subparsers.add_parser("list",
                                        help="List the installed Resilient Circuits components")
    test_parser = subparsers.add_parser("test",
                                        help="An interactive client for testing Resilient Circuits messages")
    service_parser = subparsers.add_parser("service",
                                           help="Manage Resilient Circuits as a service")
    config_parser = subparsers.add_parser("config",
                                          help="Create or update a basic configuration file")
    codegen_parser = subparsers.add_parser("codegen", parents=[common_parser],
                                           help="Generate template code for Python components")
    extractfile_parser = subparsers.add_parser("extract", parents=[common_parser],
                                                help="Extract data in order to publish a .res file")
    customize_parser = subparsers.add_parser("customize",
                                             help="Apply customizations to the Resilient platform")
    selftest_parser = subparsers.add_parser("selftest",
                                        help="Calls selftest functions for every package and prints out their return states")
    clone_parser = subparsers.add_parser("clone",
                                         help="Clone Resilient objects")
    '''Commenting out ext commands until future release
    # Add parser for ext:package
    # Usage 1: resilient-circuits ext:package <<path_to_package>>
    # Usage 2: resilient-circuits ext:package --display_name "My New Extension" <<path_to_package>>
    ext_package_help_msg = "Package an Integration into a Resilient Extension"
    ext_package_parser = subparsers.add_parser("ext:package",
                                        usage="%(prog)s -p <<path_to_package>>",
                                        help=ext_package_help_msg,
                                        description=ext_package_help_msg,
                                        argument_default=argparse.SUPPRESS)

    ext_package_parser = add_ext_arguments("ext:package", ext_package_parser)

    # Add parser for ext:convert
    # Usage: resilient-circuits ext:convert <<path_to_built_distribution>>
    ext_convert_help_msg = "Convert an old (built) Integration that can be in .tar.gz or .zip format into a Resilient Extension"
    ext_convert_parser = subparsers.add_parser("ext:convert",
                                        usage="%(prog)s -p <<path_to_built_distribution>>",
                                        help=ext_convert_help_msg,
                                        description=ext_convert_help_msg,
                                        argument_default=argparse.SUPPRESS)

    ext_convert_parser = add_ext_arguments("ext:convert", ext_convert_parser)
    '''
    # Options for selftest
    selftest_parser.add_argument("-l", "--list",
                               dest="install_list",
                               help="Test specified list of package(s)",
                               nargs="+")

    # Options for 'list'
    list_parser.add_argument("-v", "--verbose", action="store_true")

    # Options for 'config'
    file_option_group = config_parser.add_mutually_exclusive_group(required=True)
    file_option_group.add_argument("-u", "--update",
                                   help="Add any missing sections required for installed components",
                                   action="store_true")
    file_option_group.add_argument("-c", "--create",
                                   help="Create new config file with all required sections",
                                   action="store_true")
    config_parser.add_argument("filename",
                               help="Config file to write to; e.g. 'app.config'",
                               default="",
                               nargs="?")
    config_parser.add_argument("-l", "--list",
                                  dest="install_list",
                                  help="Config specified list of package(s)",
                                  nargs="+")

    # Options for 'run'
    run_parser.add_argument("-r", "--auto-restart",
                            help="Automatically restart all components if config file changes",
                            action="store_true")
    run_parser.add_argument("--config-file",
                            help="Pull configuration from specified file",
                            default=None)
    run_parser.add_argument("resilient_circuits_args", help="Args to pass to app.run", nargs=argparse.REMAINDER)

    # Options for 'service'
    service_parser.add_argument("--res-circuits-args",
                                help="Arguments to pass to resilient-circuits.exe run command",
                                action="store",
                                default="")
    service_parser.add_argument("service_args", help="Args to pass to service manager", nargs=argparse.REMAINDER)

    # Options for codegen
    codegen_parser.add_argument("-p", "--package",
                                type=ensure_unicode,
                                help="Name of the package to generate")
    codegen_parser.add_argument("--reload",
                                metavar='PACKAGE',
                                type=ensure_unicode,
                                help="Reload customizations and create new customize.py")
    # Options for extract
    extractfile_parser.add_argument("--zip",
                                    action='store_true',
                                    help="zip of the resulting file")
    # Options for 'customize'
    customize_parser.add_argument("-y",
                                  dest="yflag",
                                  help="Customize without prompting for confirmation",
                                  action="store_true")
    customize_parser.add_argument("-l", "--list",
                                  dest="install_list",
                                  help="Install specified list of package(s)",
                                  nargs="+")

    clone_parser.add_argument("--workflow",
                              help='Clone workflows. "old-api-name" "new-api-name". Workflows are based off of the'
                                   ' last export.',
                              nargs=2)

    args, unknown_args = parser.parse_known_args()
    if args.verbose:
        LOG.debug("Verbose Logging Enabled")
        LOG.setLevel(logging.DEBUG)

    if args.cmd != "run" and len(unknown_args) > 0:
        # Shouldn't have any unknown args for other commands, generate the proper errors
        args = parser.parse_args()

    if args.cmd == "run":
        run(unknown_args + args.resilient_circuits_args,
            restartable=args.auto_restart,
            config_file=args.config_file)
    elif args.cmd == "test":
        from resilient_circuits.bin import res_action_test
        res_action_test.ResilientTestProcessor().cmdloop()
    elif args.cmd == "config":
        generate_or_update_config(args)
    elif args.cmd == "list":
        list_installed(args)
    elif args.cmd == "service":
        manage_service(unknown_args + args.service_args, args.res_circuits_args)
    elif args.cmd in ("codegen", "extract"):
        if args.cmd == "codegen" and args.package is None and args.function is None and args.reload is None:
            codegen_parser.print_usage()
        elif args.cmd == "extract" and args.output is None:
            extractfile_parser.print_usage()
        else:
            logging.basicConfig(format='%(message)s', level=logging.INFO)
            generate_code(args)
    elif args.cmd == "customize":
        logging.basicConfig(format='%(message)s', level=logging.INFO)
        customize_resilient(args)
    elif args.cmd == "clone":
        if args.workflow is None:
            print('Please specify a workflow to clone')
            clone_parser.print_usage()
        else:
            clone(args)
    elif args.cmd == "selftest":
        selftest(args)

    elif "ext:" in args.cmd:
        # Call the ext: command handler
        ext_command_handler(args.cmd, args)

if __name__ == "__main__":
    LOG.debug("CALLING MAIN")
    main()



# (c) Copyright IBM Corp. 2010, 2017. All Rights Reserved.
from __future__ import print_function
import pytest
import doctest
import time
import types
import os
import resilient_circuits
import resilient_circuits.bin
import resilient_circuits.util


class TestRC:
    """Basic API tests"""

    def test_doctest(self):
        """Run doctest on all modules, even if you didn't --doctest-modules"""
        for item in resilient_circuits.__dict__.values():
            if isinstance(item, types.ModuleType):
                if 'resilient-circuits' in os.path.dirname(item.__file__):
                    doctest.testmod(item, verbose=True, raise_on_error=True)

        for item in resilient_circuits.bin.__dict__.values():
            if isinstance(item, types.ModuleType):
                doctest.testmod(item, verbose=True, raise_on_error=True)

        for item in resilient_circuits.util.__dict__.values():
            if isinstance(item, types.ModuleType):
                doctest.testmod(item, verbose=True, raise_on_error=True)



# -*- coding: utf-8 -*-
# pragma pylint: disable=unused-argument, no-self-use
"""Function implementation"""

import logging
from resilient_circuits import ResilientComponent, function, handler, StatusMessage, FunctionResult, FunctionError


class FunctionComponent(ResilientComponent):
    """Component that implements Resilient function 'mock_function_one"""

    def __init__(self, opts):
        """constructor provides access to the configuration options"""
        super(FunctionComponent, self).__init__(opts)
        self.options = opts.get("fn_mock_integration", {})

    @handler("reload")
    def _reload(self, event, opts):
        """Configuration options have changed, save new values"""
        self.options = opts.get("fn_mock_integration", {})

    @function("mock_function_one")
    def _mock_function_one_function(self, event, *args, **kwargs):
        """Function: mock_description"""
        try:
            # Get the wf_instance_id of the workflow this Function was called in
            wf_instance_id = event.message["workflow_instance"]["workflow_instance_id"]

            # Get the function parameters:
            mock_field_one = kwargs.get("mock_field_one")  # text
            mock_field_two = kwargs.get("mock_field_two")  # number

            log = logging.getLogger(__name__)
            log.info("mock_field_one: %s", mock_field_one)
            log.info("mock_field_two: %s", mock_field_two)

            # PUT YOUR FUNCTION IMPLEMENTATION CODE HERE
            #  yield StatusMessage("starting...")
            #  yield StatusMessage("done...")

            results = {
                "value": "xyz"
            }

            # Produce a FunctionResult with the results
            yield FunctionResult(results)
        except Exception:
            yield FunctionError()


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2019. All Rights Reserved.
# pylint: disable=line-too-long

""" Python Module that exposes the Ext class """

import logging
import os
import re
import hashlib
import uuid
from resilient_circuits.util.ext.ExtException import ExtException

# Get the same logger object that is used in resilient_circuits_cmd.py
LOG = logging.getLogger("resilient_circuits_cmd_logger")


class Ext(object):
    """ The is the Super class that contains methods that are shared
    among all classes that handle creating, converting, (and in the future installing)
    Extensions"""

    def __init__(self, cmd):
        # Set the command_ran in our custom ExtException
        ExtException.command_ran = cmd

    @staticmethod
    def __write_file__(path, contents):
        """Writes the String contents to a file at path"""
        # If contents is type str, encode to bytes. Needed to support PY3
        if isinstance(contents, str):
            contents = contents.encode("utf-8")
        with open(path, "wb") as the_file:
            the_file.write(contents)

    @staticmethod
    def __read_file__(path):
        """Returns all the lines of a file at path as a List"""
        file_lines = []
        with open(path, "r") as the_file:
            file_lines = the_file.readlines()
        return file_lines

    @staticmethod
    def __has_permissions__(permissions, path):
        """Raises an exception if the user does not have the given permissions to path"""

        if not os.access(path, permissions):

            if permissions is os.R_OK:
                permissions = "READ"
            elif permissions is os.W_OK:
                permissions = "WRITE"

            raise ExtException("User does not have {0} permissions for: {1}".format(permissions, path))

    @staticmethod
    def __validate_directory__(permissions, path_to_dir):
        """Check the given path is absolute, exists and has the given permissions, else raises an Exception"""

        # Check the path is absolute
        if not os.path.isabs(path_to_dir):
            raise ExtException("The path to the directory must be an absolute path: {0}".format(path_to_dir))

        # Check the directory exists
        if not os.path.isdir(path_to_dir):
            raise ExtException("The path does not exist: {0}".format(path_to_dir))

        # Check we have the correct permissions
        Ext.__has_permissions__(permissions, path_to_dir)

    @staticmethod
    def __validate_file_paths__(permissions=None, *args):
        """Check the given *args paths exist and has the given permissions, else raises an Exception"""

        # For each *args
        for path_to_file in args:
            # Check the file exists
            if not os.path.isfile(path_to_file):
                raise ExtException("Could not find file: {0}".format(path_to_file))

            if permissions:
                # Check we have the correct permissions
                Ext.__has_permissions__(permissions, path_to_file)

    @staticmethod
    def __is_valid_url__(url):
        """Returns True if url is valid, else False. Accepted url examples are:
            "http://www.example.com", "https://www.example.com", "www.example.com", "example.com" """

        if not url:
            return False

        regex = re.compile(
            r'^(https?://)?'  # optional http:// or https://
            r'(?:(?:[A-Z0-9](?:[A-Z0-9-]{0,61}[A-Z0-9])?\.)+[A-Z]{2,6}\.?)'  # domain/hostname
            r'(?:/?|[/?]\S+)$',  # .com etc.
            re.IGNORECASE)

        return regex.search(url) is not None

    @staticmethod
    def __is_valid_package_name__(name):
        """Returns True if name is valid, else False. Accepted name examples are:
            "fn_my_new_package" """

        if not name:
            return False

        regex = re.compile(r'^[0-9A-Z_]+$', re.IGNORECASE)

        return regex.match(name) is not None

    @staticmethod
    def __is_valid_version_syntax__(version):
        """Returns True if version is valid, else False. Accepted version examples are:
            "1.0.0" "1.1.0" "123.0.123" """
        if not version:
            return False

        regex = re.compile(r'^[0-9]+\.[0-9]+\.[0-9]+$')

        return regex.match(version) is not None

    @staticmethod
    def __generate_uuid_from_string__(the_string):
        """Returns String representation of the UUID of a hex md5 hash of the given string"""

        # Instansiate new md5_hash
        md5_hash = hashlib.md5()

        # Pass the_string to the md5_hash as bytes
        md5_hash.update(the_string.encode("utf-8"))

        # Generate the hex md5 hash of all the read bytes
        the_md5_hex_str = md5_hash.hexdigest()

        # Return a String repersenation of the uuid of the md5 hash
        return str(uuid.UUID(the_md5_hex_str))

    @staticmethod
    def __generate_md5_uuid_from_file__(path_to_file):
        """Returns String representation of the UUID of a hex md5 hash of the given file"""

        # Instansiate new md5_hash
        md5_hash = hashlib.md5()

        # Open a stream to the file. Read 4096 bytes at a time. Pass these bytes to md5_hash object
        with open(path_to_file, mode="rb") as the_file:
            for chunk in iter(lambda: the_file.read(4096), b''):
                md5_hash.update(chunk)

        # Generate the hex md5 hash of all the read bytes
        the_md5_hex_str = md5_hash.hexdigest()

        # Return a String repersenation of the uuid of the md5 hash
        return str(uuid.UUID(the_md5_hex_str))



# (c) Copyright IBM Corp. 2010, 2018. All Rights Reserved.

from .resilient_customize import \
    TypeDefinition, \
    MessageDestinationDefinition, \
    ActionDefinition, \
    FunctionDefinition, \
    ImportDefinition

from .resilient_codegen import list_functions, valid_identifier
from .resilient_config import get_config_data
from .resilient_customize import get_customization_definitions, get_function_definition



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2019. All Rights Reserved.
# pylint: disable=line-too-long

""" Python Module that exposes the ExtPackage class """

import logging
import os
from setuptools import sandbox as use_setuptools
from resilient_circuits.util.ext.ExtCreate import ExtCreate

# Get the same logger object that is used in resilient_circuits_cmd.py
LOG = logging.getLogger("resilient_circuits_cmd_logger")

# Constants
BASE_NAME_SETUP_PY = "setup.py"
BASE_NAME_DIST_DIR = "dist"

PATH_CUSTOMIZE_PY = os.path.join("util", "customize.py")
PATH_CONFIG_PY = os.path.join("util", "config.py")

PATH_ICON_EXTENSION_LOGO = os.path.join("icons", "extension_logo.png")
PATH_ICON_COMPANY_LOGO = os.path.join("icons", "company_logo.png")


class ExtPackage(ExtCreate):
    """ ExtPackage is a subclass of ExtCreate. It exposes one
    method: package_extension() """

    @classmethod
    def package_extension(cls, path_to_src, custom_display_name=None, keep_build_dir=False):
        """ Function that creates The Extension.zip file from the give source path and returns
        the path to the new Extension.zip
        - path_to_src [String]: must include a setup.py, customize.py and config.py file.
        - custom_display_name [String]: will give the Extension that display name. Default: name from setup.py file
        - keep_build_dir [Boolean]: if True, dist/build/ will not be remove. Default: False
        - The code will be packaged into a Built Distribution (.tar.gz) in the /dist directory
        - The Extension.zip will also be produced in the /dist directory"""

        # Ensure the src directory exists and we have WRITE access
        cls.__validate_directory__(os.W_OK, path_to_src)

        # Generate paths to files required to create extension
        path_setup_py_file = os.path.join(path_to_src, BASE_NAME_SETUP_PY)
        path_customize_py_file = os.path.join(path_to_src, os.path.basename(path_to_src), PATH_CUSTOMIZE_PY)
        path_config_py_file = os.path.join(path_to_src, os.path.basename(path_to_src), PATH_CONFIG_PY)
        path_output_dir = os.path.join(path_to_src, BASE_NAME_DIST_DIR)
        path_extension_logo = os.path.join(path_to_src, PATH_ICON_EXTENSION_LOGO)
        path_company_logo = os.path.join(path_to_src, PATH_ICON_COMPANY_LOGO)

        LOG.info("Creating Built Distribution in /dist directory")

        # Create the built distribution
        use_setuptools.run_setup(setup_script=path_setup_py_file, args=["sdist", "--formats=gztar"])

        LOG.info("Built Distribution (.tar.gz) created at: %s", path_output_dir)

        # Create the extension
        path_the_extension_zip = cls.create_extension(
            path_setup_py_file=path_setup_py_file,
            path_customize_py_file=path_customize_py_file,
            path_config_py_file=path_config_py_file,
            output_dir=path_output_dir,
            custom_display_name=custom_display_name,
            keep_build_dir=keep_build_dir,
            path_extension_logo=path_extension_logo,
            path_company_logo=path_company_logo
        )

        LOG.info("Extension created at: %s", path_the_extension_zip)

        return path_the_extension_zip



# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2018. All Rights Reserved.

"""Utility to codegen a resilient-circuits component or package"""

from __future__ import print_function
import os
import sys
import io
import json
import logging
import keyword
import re
import datetime
import uuid
import time
import copy
import pkg_resources
from zipfile import ZipFile
from resilient_circuits import template_functions

if sys.version_info.major == 2:
    import imp
else:
    import importlib.util

LOG = logging.getLogger("__name__")


# A (const, random) namespace for import
UUID_CODEGEN = uuid.UUID('bfeec2d4-3770-11e8-ad39-4a0004044aa0')

# Template files that drive codegen
FUNCTION_TEMPLATE_PATH = "data/template_function.jinja2"
PACKAGE_TEMPLATE_PATH = "data/template_package.jinja2"

INCIDENT_TYPE_ID = 0
ACTION_TYPE_ID = 6
DATATABLE_TYPE_ID = 8
FUNCTION_TYPE_ID = 11

# The attributes we want to keep from the object definitions
TEMPLATE_ATTRIBUTES = [
    "template",
    "name"
]

VALUE_ATTRIBUTES = [
    "label"
]

FUNCTION_FIELD_ATTRIBUTES = [
    "uuid",  # note - workflows reference function inputs by uuid
    "templates",
    "text",
    "tooltip",
    "rich_text",
    "values",
    "blank_option",
    "input_type",
    "placeholder",
    "required",
    "name"
]

ACTION_FIELD_ATTRIBUTES = FUNCTION_FIELD_ATTRIBUTES
INCIDENT_FIELD_ATTRIBUTES = FUNCTION_FIELD_ATTRIBUTES
DATATABLE_FIELD_ATTRIBUTES = FUNCTION_FIELD_ATTRIBUTES

VIEW_ITEM_ATTRIBUTES = [
    "element",
    "content",
    "field_type"
]

FUNCTION_ATTRIBUTES = [
    "uuid",  # note - workflows reference functions by uuid
    "display_name",
    "view_items",
    "name",
    "description",
    "destination_handle"
]

AUTOMATIC_TASK_ATTRIBUTES = [
    "uuid",  # note - workflows reference tasks by uuid
    "due_date_offset",
    "due_date_units",
    "name",
    "enabled",
    "task_layout",
    "programmatic_name",
    "phase_id",
    "optional",
    "instructions"
]

AUTOMATIC_TASK_LAYOUT_ATTRIBUTES = [
    "show_if",
    "field_type",
    "show_link_header",
    "element",
    "content",
    "step_label"
]

PHASE_ATTRIBUTES = [
    "uuid",
    "name",
    "enabled",
    "order"
]

SCRIPT_ATTRIBUTES = [
    "uuid",  # note - workflows reference scripts by uuid
    "description",
    "language",
    "object_type",
    "name",
    "script_text"
]

WORKFLOW_ATTRIBUTES = [
    "programmatic_name",
    "object_type",
    "content"
]

WORKFLOW_CONTENT_ATTRIBUTES = [
    "xml"
]

MESSAGE_DESTINATION_ATTRIBUTES = [
    "programmatic_name",
    "name",
    "expect_ack",
    "destination_type"
]

ACTION_ATTRIBUTES = [
    "logic_type",
    "name",
    "view_items",
    "type",
    "workflows",
    "object_type",
    "timeout_seconds",
    "automations",
    "conditions",
    "message_destinations"
]

ARTIFACT_TYPE_ATTRIBUTES = [
    "uuid",
    "desc",
    "name",
    "programmatic_name",
    "file",
    "system",
    "multi_aware",
    "use_for_relationships",
    "parse_for_csv",
    "enabled"
]

def valid_identifier(name):
    """Test if 'name' is a valid identifier for a package or module

       >>> valid_identifier("")
       False
       >>> valid_identifier("get")
       False
       >>> valid_identifier("bang!")
       False
       >>> valid_identifier("_something")
       True
    """
    if keyword.iskeyword(name):
        return False
    if name in dir(__builtins__):
        return False
    return re.match("[_A-Za-z][_a-zA-Z0-9]*$", name) is not None


def list_functions(function_defs):
    """List all the functions"""
    print(u"Available functions:")
    for function_def in function_defs:
        print(u"    {}".format(function_def["name"]))


def list_workflows(workflow_defs):
    """List all the workflows"""
    print(u"Available workflows:")
    for workflow_def in workflow_defs:
        print(u"    {}".format(workflow_def["programmatic_name"]))


def list_actions(action_defs):
    """List all the actions (rules)"""
    print(u"Available rules:")
    for action_def in action_defs:
        print(u"    {}".format(action_def["name"]))


def list_message_destinations(message_destination_defs):
    """List all the message destinations"""
    print(u"Available message destinations:")
    for message_destination_def in message_destination_defs:
        print(u"    {}".format(message_destination_def["programmatic_name"]))


def list_incident_fields(field_defs):
    """List all the custom incident fields"""
    print(u"Available incident fields:")
    for field_def in field_defs:
        if field_def["type_id"] == INCIDENT_TYPE_ID and field_def.get("prefix") == "properties":
            print(u"    {}".format(field_def["name"]))


def list_datatables(datatable_defs):
    """List all the datatables"""
    print(u"Available datatables:")
    for datatable_def in datatable_defs:
        if datatable_def["type_id"] == DATATABLE_TYPE_ID:
            print(u"    {}".format(datatable_def["type_name"]))


def list_automatic_tasks(task_defs):
    """List all the tasks (built-in and custom are not distinguished)"""
    print(u"Available tasks:")
    for task_def in task_defs:
        print(u"    {}".format(task_def["programmatic_name"]))


def list_scripts(script_defs):
    """List all the scripts"""
    print(u"Available scripts:")
    for script_def in script_defs:
        print(u"    {}".format(script_def["name"]))

def list_artifact_types(artifact_type_defs):
    """List all the artifact types"""
    print(u"Available artifact types:")
    for artifact_type_def in artifact_type_defs:
        print(u"    {}".format(artifact_type_def["name"]))


def clean(dictionary, keep):
    """Remove attributes that are not in the 'keep' list"""
    for key in dictionary.copy().keys():
        if key not in keep:
            dictionary.pop(key)
    return dictionary


def render_file_mapping(file_mapping_dict, data, source_dir, target_dir):
    """
    Walk each value in the "rendered" file-mapping dictionary, and create the target files.

    Nesting in the 'target' dictionary represents the target directory structure.
    Source values are the full path to a source file.
    Each source file is treated as a JINJA2 template, and rendered using the data provided.

    :param file_mapping_dict: {"target": "source"...}
    :param data: the data for JINJA rendering of each source file
    :param source_dir: path to the root of the source files
    :param target_dir: path where the target files and directories should be written
    """
    for (key, value) in sorted(file_mapping_dict.items()):
        if not key:
            LOG.error(u"Cannot render empty target for %s", value)
            continue
        # The key is a directory-name or filename,
        # optionally followed by a '@xxx" where 'xxx' is a variable tha the
        # template needs, such as a loop-variable.  Split this out if present.
        loopvar = None
        if "@" in key:
            split = key.split("@", 1)
            key = split[0]
            loopvar = split[1]
        data["loopvar"] = loopvar
        #
        if isinstance(value, dict):
            # This is a subdirectory
            subdir = os.path.join(target_dir, key)
            try:
                os.mkdir(subdir)
            except OSError as exc:
                LOG.warning(exc)
            render_file_mapping(value, data, source_dir, subdir)
        else:
            target_file = os.path.join(target_dir, key)
            source_file = os.path.join(source_dir, value)
            if os.path.exists(target_file):
                LOG.error(u"Not writing %s: file exists.", target_file)
                continue
            # Render the source file as a JINJA template
            LOG.debug(u"Writing %s from template %s", target_file, source_file)
            LOG.info(u"Writing %s", target_file)

            # Handle if the file is a .png
            if os.path.splitext(source_file)[1] == ".png":
                with io.open(source_file, 'rb') as source:
                    png_as_bytes = source.read()
                with io.open(target_file, mode="wb") as outfile:
                    outfile.write(png_as_bytes)

            else:
                with io.open(source_file, mode='r', encoding="utf-8") as source:
                    source_template = source.read()
                    source_rendered = template_functions.render(source_template, data)
                with io.open(target_file, mode="w", encoding="utf-8") as outfile:
                    outfile.write(source_rendered)


def codegen_from_template(cmd, client, export_file, template_file_path, package,
                          message_destination_names, function_names, workflow_names, action_names,
                          field_names, datatable_names, task_names, script_names, artifact_type_names,
                          output_dir, output_file, zip):
    """Based on a template-file, produce the generated file or package.

       To codegen a single file, the template will be a JSON dict with just one entry,
       such as {"file_to_generate.py": "path/to/template.jinja2"}
       To codegen a whole directory, the template dict can have multiple values,
       including nested subdirectories.

       Each source ("path/to/template.jinja2") will be rendered using jinja2,
       then written to the target ("file_to_generate.py").

       :param cmd: 'codegen' or 'extract'
       :param client: the REST client
       :param export_file: file containing customization exports (default is to use the server's latest)
       :param template_file_path: location of templates
       :param package: name of the package to be generated
       :param message_destination_names: list of message desctinations; generate all the functions that use them
       :param function_names: list of named functions to be generated
       :param workflow_names: list of workflows whose customization def should be included in the package
       :param action_names: list of actions (rules) whose customization def should be included in the package
       :param field_names: list of incident fields whose customization def should be included in the package
       :param datatable_names: list of data tables whose customization def should be included in the package
       :param task_names: list of automatic tasks whose customization def should be included in the package
       :param script_names: list of scripts whose customization def should be included in the package
       :param artifact_type_names: lists of custom artifact groups to include
       :param output_dir: output location
       :param output_file: output file name, also .res file produced for 'extract'
       :param zip: True if resulting file(s) should be zipped up
    """
    functions = {}
    function_params = {}
    message_destinations = {}
    incident_fields = {}
    action_fields = {}
    datatables = {}
    datatable_fields = {}
    phases = {}
    automatic_tasks = {}
    scripts = {}
    workflows = {}
    actions = {}
    artifact_types = {}

    if export_file:
        with io.open(export_file, 'r', encoding="utf-8") as export:
            export_data = json.loads(export.read())
        LOG.info(u"{} is based on the organization export from '{}'.".format(cmd, export_file))
    else:
        # Force a recent export
        latest_export_uri = "/configurations/exports/"
        client.post(latest_export_uri, {"layouts": True, "actions": True, "phases_and_tasks": True})

        # Get the most recent org export that includes actions and tasks
        export_uri = "/configurations/exports/history"
        export_list = client.get(export_uri)["histories"]
        last_date = 0
        last_id = 0
        for export in export_list:
            if export["options"]["actions"] and export["options"]["phases_and_tasks"]:
                if export["date"] > last_date:
                    last_date = export["date"]
                    last_id = export["id"]
        if last_date == 0:
            LOG.error(u"ERROR: No suitable export is available.  "
                      u"Create an export for code generation. (Administrator Settings -> Organization -> Export).")
            return
        dt = datetime.datetime.utcfromtimestamp(last_date/1000.0)
        LOG.info(u"{} is based on the organization export from {}.".format(cmd, dt))
        export_uri = "/configurations/exports/{}".format(last_id)
        export_data = client.get(export_uri)

    # Get definitions for custom incident fields - used in multiple areas
    all_fields = dict((field["name"], field)
                      for field in export_data.get("fields")
                      if field["type_id"] == INCIDENT_TYPE_ID and field.get("prefix") == "properties")

    all_destinations = dict((dest["programmatic_name"], dest)
                            for dest in export_data.get("message_destinations", []))
    all_destinations_2 = dict((dest["name"], dest)
                              for dest in export_data.get("message_destinations", []))

    if function_names or message_destination_names:
        # Check that 'functions' are available (v30 onward)
        function_defs = export_data.get("functions")
        if not function_defs:
            LOG.error(u"ERROR: Functions are not available in this export.")
            return
        function_names = function_names or []
        available_names = [function_def["name"] for function_def in function_defs]
        if message_destination_names:
            # Build a list of all the functions that use the specified message destination(s)
            for function_def in function_defs:
                if function_def["destination_handle"] in message_destination_names:
                    function_names.append(function_def["name"])

        # Check that each named function is available
        for function_name in function_names or []:
            if function_name not in available_names:
                LOG.error(u"ERROR: Function '%s' not found in this export.", function_name)
                list_functions(function_defs)
                return

        # Check that the named message destination is available
        for message_destination_name in message_destination_names or []:
            if message_destination_name not in all_destinations:
                LOG.error(u"ERROR: Message destination '%s' not found in this export.", message_destination_name)
                list_message_destinations(export_data.get("message_destinations"))
                return

    if workflow_names:
        # Check that 'workflows' are available (v28 onward)
        workflow_defs = export_data.get("workflows")
        if not workflow_defs:
            LOG.error(u"ERROR: Workflows are not available in this export.")
            return
    else:
        workflow_names = []

    # ensure empty lists as we may dynamically add to them
    if not task_names:
        task_names = []
    if not script_names:
        script_names = []
    if not artifact_type_names:
        artifact_type_names = []
    if not field_names:
        field_names = []

    if action_names:
        # Check that 'actions' are available. actions are rules
        action_defs = export_data.get("actions")
        if not action_defs:
            LOG.error(u"ERROR: Rules are not available in this export.")
            return

        # Check that each named action is available
        actions = {action_def["name"]: clean(copy.deepcopy(action_def), ACTION_ATTRIBUTES)
                   for action_def in action_defs
                   if action_def["name"] in action_names}
        all_action_fields = dict((field["uuid"], field)
                                 for field in export_data.get("fields")
                                 if field["type_id"] == ACTION_TYPE_ID)
        all_action_fields_2 = dict((field["name"], field)
                                 for field in export_data.get("fields")
                                 if field["type_id"] == ACTION_TYPE_ID)

        for action_name in action_names:
            if action_name not in actions:
                LOG.error(u"ERROR: Rule '%s' not found in this export.", action_name)
                list_actions(action_defs)
                return
            action_def = actions[action_name]

            # Get the activity-fields for this action (if any)
            action_field_uuids = [item.get("content")
                                  for item in action_def["view_items"]
                                  if "content" in item and item.get("field_type")]
            fields = []
            for field_uuid in action_field_uuids:
                field = copy.deepcopy(all_action_fields.get(field_uuid))
                if field is None:
                    # v29-style export where layout indexed by field name
                    field = copy.deepcopy(all_action_fields_2.get(field_uuid))
                clean(field, ACTION_FIELD_ATTRIBUTES)
                for template in field.get("templates", []):
                    clean(template, TEMPLATE_ATTRIBUTES)
                for value in field.get("values", []):
                    clean(value, VALUE_ATTRIBUTES)
                fields.append(field)
                action_fields[field["name"]] = field

            # Get the workflow(s) for this rule (if any)
            wf_names = action_def["workflows"]
            for wf_name in wf_names:
                if wf_name not in workflow_names:
                    workflow_names.append(wf_name)

            # Get the task(s) for this rule (if any)
            for automation in action_def["automations"]:
                if automation.get("tasks_to_create"):
                    for task_name in automation["tasks_to_create"]:
                        if task_name not in task_names:
                            task_names.append(task_name)

                elif automation.get("scripts_to_run"):
                    script_name = automation["scripts_to_run"]
                    if script_name not in script_names:
                        script_names.append(script_name)

                elif automation.get("field"):
                    field_name = automation["field"]
                    if field_name not in field_names and field_name in all_fields:
                        field_names.append(field_name)

            # Get the message destination(s) for this rule (if any)
            dest_names = action_def["message_destinations"]
            for dest_name in dest_names:
                if dest_name not in message_destinations:
                    dest = copy.deepcopy(all_destinations_2[dest_name])
                    clean(dest, MESSAGE_DESTINATION_ATTRIBUTES)
                    message_destinations[dest_name] = dest

    all_functions = dict((function["name"], function)
                         for function in export_data.get("functions") or [])
    all_function_fields = dict((field["uuid"], field)
                               for field in export_data.get("fields")
                               if field["type_id"] == FUNCTION_TYPE_ID)

    for function_name in (function_names or []):
        # Get the function definition
        function_def = copy.deepcopy(all_functions.get(function_name))
        # Remove the attributes we don't want to serialize
        clean(function_def, FUNCTION_ATTRIBUTES)
        for view_item in function_def.get("view_items", []):
            clean(view_item, VIEW_ITEM_ATTRIBUTES)
        functions[function_name] = function_def

        # Get the parameters (input fields) for this function
        param_names = [item.get("content")
                       for item in function_def["view_items"]
                       if "content" in item]
        params = []
        for param_name in param_names:
            param = copy.deepcopy(all_function_fields[param_name])
            clean(param, FUNCTION_FIELD_ATTRIBUTES)
            for template in param.get("templates", []):
                clean(template, TEMPLATE_ATTRIBUTES)
            for value in param.get("values", []):
                clean(value, VALUE_ATTRIBUTES)
            params.append(param)
            function_params[param["uuid"]] = param

        # Get the message destination for this function
        dest_name = function_def["destination_handle"]
        if dest_name not in message_destinations:
            dest = copy.deepcopy(all_destinations[dest_name])
            clean(dest, MESSAGE_DESTINATION_ATTRIBUTES)
            message_destinations[dest_name] = dest

    if workflow_names:
        all_workflows = dict((workflow["programmatic_name"], workflow)
                             for workflow in export_data.get("workflows"))
        for workflow_name in workflow_names:
            # Get the workflow definition
            workflow_def = all_workflows.get(workflow_name)
            if workflow_def:
                # Remove the attributes we don't want to serialize
                workflow = clean(copy.deepcopy(workflow_def), WORKFLOW_ATTRIBUTES)
                clean(workflow["content"], WORKFLOW_CONTENT_ATTRIBUTES)
                workflows[workflow_name] = workflow
            else:
                LOG.error(u"ERROR: Workflow '%s' not found in this export.", workflow_name)
                list_workflows(export_data.get("workflows"))
                return

    if field_names:
        for field_name in field_names:
            fielddef = all_fields.get(field_name)
            if fielddef:
                field = clean(copy.deepcopy(fielddef), INCIDENT_FIELD_ATTRIBUTES)
                for template in field.get("templates", []):
                    clean(template, TEMPLATE_ATTRIBUTES)
                for value in field.get("values", []):
                    clean(value, VALUE_ATTRIBUTES)
                incident_fields[field["uuid"]] = field
            else:
                LOG.error(u"ERROR: Custom incident field '%s' not found in this export.", field_name)
                list_incident_fields(export_data.get("fields"))
                return

    if datatable_names:
        # Get datatable definitions
        all_datatables = dict((table["type_name"], table)
                              for table in export_data.get("types")
                              if table["type_id"] == DATATABLE_TYPE_ID)
        for datatable_name in datatable_names:
            datatable = all_datatables.get(datatable_name)
            if datatable:
                for (fieldname, fielddef) in datatable["fields"].items():
                    field = clean(copy.deepcopy(fielddef), DATATABLE_FIELD_ATTRIBUTES)
                    for template in field.get("templates", []):
                        clean(template, TEMPLATE_ATTRIBUTES)
                    for value in field.get("values", []):
                        clean(value, VALUE_ATTRIBUTES)
                    datatable_fields[field["uuid"]] = field
                datatables[datatable_name] = datatable
            else:
                LOG.error(u"ERROR: Datatable '%s' not found in this export.", datatable_name)
                list_datatables(export_data.get("types", []))
                return

    # Automatic tasks determine the list of phases
    phase_names = set()
    if task_names:
        # Get task definitions
        all_tasks = dict((task["programmatic_name"], task)
                          for task in export_data.get("automatic_tasks"))
        for task_name in task_names:
            task = all_tasks.get(task_name)
            if task:
                automatic_tasks[task_name] = clean(copy.deepcopy(task), AUTOMATIC_TASK_ATTRIBUTES)
                phase_names.add(task["phase_id"])
            else:
                LOG.error(u"ERROR: Task '%s' not found in this export.", task_name)
                list_automatic_tasks(export_data.get("automatic_tasks", []))
                return

    if phase_names:
        # Get phase definitions
        all_phases = dict((phase["name"], phase)
                          for phase in export_data.get("phases"))
        for phase_name in phase_names:
            # Assume phase-name is found.  It was derived from the automatic task.
            phase = all_phases[phase_name]
            phases[phase_name] = clean(copy.deepcopy(phase), PHASE_ATTRIBUTES)

    if script_names:
        # Get script definitions
        all_scripts = dict((script["name"], script)
                           for script in export_data.get("scripts"))
        for script_name in script_names:
            script = all_scripts.get(script_name)
            if script:
                scripts[script_name] = clean(copy.deepcopy(script), SCRIPT_ATTRIBUTES)
            else:
                LOG.error(u"ERROR: Script '%s' not found in this export.", script_name)
                list_scripts(export_data.get("scripts", []))
                return

    if artifact_type_names:
        # get custom artifact types
        all_artifact_types = dict((artifact_type["programmatic_name"], artifact_type)
                          for artifact_type in export_data.get("incident_artifact_types"))

        for artifact_type_name in artifact_type_names:
            for artifact_name in all_artifact_types:
                artifact = all_artifact_types[artifact_name]
                if artifact["programmatic_name"] == artifact_type_name:
                    artifact_types[artifact["programmatic_name"]] = clean(copy.deepcopy(artifact), ARTIFACT_TYPE_ATTRIBUTES)

        # confirm we got all the custom artifact types
        for artifact_name in artifact_type_names:
            if artifact_name not in artifact_types:
                LOG.error(u"ERROR: Artifact Type '%s' not found in this export.", artifact_name)
                list_artifact_types(export_data.get("incident_artifact_types", []))
                return

    # Minify the export_data
    fields_list = []
    if len(incident_fields) == 0:
        # import requires at least one, use placeholder
        fields_list.extend(["incident/inc_training"])
    else:
        fields_list.extend(["incident/{}".format(fld["name"]) for fld in incident_fields.values()])
    fields_list.extend(["actioninvocation/{}".format(fld["name"]) for fld in action_fields.values()])
    fields_list.extend(["__function/{}".format(fld["name"]) for fld in function_params.values()])
    keep_keys = [
        "export_date",
        "export_format_version",
        "id",
        "server_version"
    ]
    minify_keys = {
        "actions": {"name": actions.keys()},
        "automatic_tasks": {"programmatic_name": automatic_tasks.keys()},
        "fields": {"export_key": fields_list},
        "functions": {"name": functions.keys()},
        "message_destinations": {"programmatic_name": message_destinations.keys()},
        "phases": {"name": phases.keys()},
        "scripts": {"name": scripts.keys()},
        "types": {"type_name": datatables.keys()},
        "workflows": {"programmatic_name": workflows.keys()},
        "incident_artifact_types": {"programmatic_name": artifact_types.keys()}
    }
    for key in export_data.keys():
        if key in keep_keys:
            pass
        elif key in minify_keys.keys():
            name = list(minify_keys[key].keys())[0]   # The property we match on
            values = minify_keys[key][name]           # These are the names of the things to keep
            for data in list(export_data[key]):
                if not data.get(name):
                    LOG.warning("No %s in %s", name, key)
                if not data.get(name) in values:
                    export_data[key].remove(data)
        elif isinstance(export_data[key], list):
            export_data[key] = []
        elif isinstance(export_data[key], dict):
            export_data[key] = {}
        else:
            export_data[key] = None
    # Incident types are special, add one for this specific package
    # (because not enabled, this doesn't actually get loaded into the destination)
    t0 = int(time.time()*1000)
    export_data["incident_types"] = [{
        "update_date": t0,
        "create_date": t0,
        "uuid": str(UUID_CODEGEN),
        "description": "Customization Packages (internal)",
        "export_key": "Customization Packages (internal)",
        "name": "Customization Packages (internal)",
        "enabled": False,
        "system": False,
        "parent_id": None,
        "hidden": False,
        "id": 0
    }]

    # if an extract, write the file and return
    if cmd == "extract":
        do_extract(output_file, export_data, zip)
        return

    # Prepare the dictionary of substitution values for jinja2
    # (includes all the configuration elements related to the functions)
    data = {
        "package": package,
        "function_names": function_names,
        "output_dir": output_dir,
        "output_file": output_file,
        "functions": functions,
        "function_params": function_params,
        "message_destinations": message_destinations,
        "incident_fields": incident_fields,
        "action_fields": action_fields,
        "datatables": datatables,
        "datatable_fields": datatable_fields,
        "phases": phases,
        "automatic_tasks": automatic_tasks,
        "scripts": scripts,
        "workflows": workflows,
        "actions": actions,
        "export_data": export_data,
        "incident_artifact_types": artifact_types
    }
    LOG.debug(u"Configuration data:\n%s", json.dumps(data, indent=2))

    # Read the files/package template and render it
    # to produce the file-mapping dictionary from template-files to generated-files
    with io.open(template_file_path, 'r', encoding="utf-8") as template_file:
        file_mapping_template = template_file.read()
        file_mapping = template_functions.render_json(file_mapping_template, data)

    LOG.debug(u"Codegen template:\n%s", json.dumps(file_mapping, indent=2))

    # Write all the files defined in the mapping definition
    src_dir = os.path.dirname(template_file_path)
    render_file_mapping(file_mapping, data, src_dir, output_dir)

def do_extract(output_file, export_data, zip):
    """
    either create the export file in clear text or zip up
    :param output_file:
    :param export_data:
    :param zip: True if zipping up file
    :return: None
    """
    if sys.version_info.major >= 3:
        res_data = json.dumps(export_data, ensure_ascii=False)
    else:
        res_data = unicode(json.dumps(export_data, ensure_ascii=False))

    if zip:
        # get the base name of the file to create so we know what to call our file inside the zip file
        if not output_file.endswith(".zip"):
            output_file = "".join((output_file, ".zip"))
        print (u"Writing {}".format(output_file))

        base = os.path.basename(output_file)
        filename = os.path.splitext(base)
        while '.' in filename[0]:
            filename = os.path.splitext(filename[0])
        # name internal to zip file
        filename = "".join((filename[0], ".res"))

        with ZipFile(output_file, 'w') as myZip:
            myZip.writestr(filename, res_data)
    else:
        print (u"Writing {}".format(output_file))
        with io.open(output_file, 'w', encoding="utf-8") as extract_fh:
            extract_fh.write(res_data)

    return

def codegen_package(client, export_file, package,
                    message_destination_names, function_names, workflow_names, action_names,
                    field_names, datatable_names, task_names, script_names, artifact_type_names,
                    output_dir):
    """Generate a an installable python package"""
    if not valid_identifier(package):
        LOG.error(u"ERROR: %s is not a valid package name.", package)
        return

    # Make the output directory (usually a new subdirectory of cwd)
    try:
        os.makedirs(output_dir)
    except OSError as exc:
        LOG.warning(u"%s", exc)

    template_file_path = pkg_resources.resource_filename("resilient_circuits", PACKAGE_TEMPLATE_PATH)
    return codegen_from_template('codegen', client, export_file, template_file_path, package,
                                 message_destination_names, function_names, workflow_names, action_names,
                                 field_names, datatable_names, task_names, script_names, artifact_type_names,
                                 output_dir, None, False)


def codegen_functions(client, export_file, function_names, workflow_names, action_names, artifact_type_names,
                      output_dir, output_file):
    """Generate a python file that implements one or more functions"""
    message_destination_names = None
    template_file_path = pkg_resources.resource_filename("resilient_circuits", FUNCTION_TEMPLATE_PATH)
    return codegen_from_template('codegen', client, export_file, template_file_path, None,
                                 message_destination_names, function_names, workflow_names, action_names,
                                 None, None, None, None, artifact_type_names,
                                 output_dir, output_file, False)

def extract_to_res(client, export_file,
                     message_destination_names, function_names, workflow_names, action_names,
                     field_names, datatable_names, task_names, script_names, artifact_types,
                     output_file, zip):
    """
    extract portions of a .res file as directed by the parameters below and save to a specified file
    :param client:
    :param export_file: .res to use otherwise the most recent one produced will be used
    :param message_destination_names:
    :param function_names:
    :param workflow_names:
    :param action_names:
    :param field_names:
    :param datatable_names:
    :param task_names:
    :param script_names:
    :param artifact_types:
    :param output_file: file to produce or overwrite
    :param zip: true if resulting file should be zipped
    :return: None
    """
    return codegen_from_template('extract', client, export_file, None, None,
                                 message_destination_names, function_names, workflow_names, action_names,
                                 field_names, datatable_names, task_names, script_names, artifact_types,
                                 None, output_file, zip)

def get_customize_file_path(package):
    """Get the location of current customize.py for this package"""
    output_base = os.path.join(os.getcwd(), package)
    customize_dir = os.path.join(output_base, package, "util")
    customize_file = os.path.join(customize_dir, "customize.py")
    return customize_file, output_base, customize_dir

def get_codegen_reload_data(package):
    """Read the default codegen_reload_data section from the given package"""

    # Get the file path of the customize file for the package
    customize_file, base_dir, customize_dir = get_customize_file_path(package)

    # Check if customize.py exits.  We need to get the reload commands
    # from the current customize.py and if it's not there then exit.
    if not os.path.isfile(customize_file):
        raise Exception(u"{} does not exist. Run resilient_circuits codegen without --reload option to create it.".format(customize_file))

    data = None

    # Dynamically load the customize module and call the codegen_reload_date routine.
    # Different pacakges are used in Python 2 and Python 3.
    if sys.version_info.major == 2:      # Python 2
        try:
            customize_module = imp.load_source("customize", customize_file)
            data = customize_module.codegen_reload_data()
        except Exception as e:
            LOG.error(u"Error loading codegen_reload_data: %s", e)
    else:                                # Python 3
        try:
            spec = importlib.util.spec_from_file_location("codegen_reload_data", customize_file)
            customize_module = importlib.util.module_from_spec(spec)
            spec.loader.exec_module(customize_module)
            data = customize_module.codegen_reload_data()
        except Exception as e:
            LOG.error(u"Error loading codegen_reload_data for package %s", e)
    return data or []


def merge_codegen_params(reload_list, arg_list):
    """Merge the codegen reload params list additional arguments list with no duplicates"""

    if reload_list:
        new_reload_list = reload_list
    else:
        new_reload_list = []

    if arg_list:
        new_arg_list = arg_list
    else:
        new_arg_list = []

    # Combine the reload and new argument list without duplicates
    combined_args_list = list(set(new_reload_list).union(set(new_arg_list)))

    return combined_args_list



def codegen_reload_package(client, args):
    """Generate a package using previous codegen parameters and add any new ones from the commandline."""

    # Get the previous params for codegen from the customize.py
    # codegen_reload_data function.
    codegen_params = get_codegen_reload_data(args.reload)

    if codegen_params == None or codegen_params == []:
        raise Exception(u"codegen_reload_data entry point returned empty list")

    # Rename the old customize.py file to customize-yyyymmdd-hhmmss.bak
    customize_file, output_base, customize_dir = get_customize_file_path(args.reload)

    # Get time now.
    now = datetime.datetime.now().strftime("%Y%m%d%H%M%S")
    old_customize_file = os.path.join(customize_dir, "customize-{}.bak".format(now))
    LOG.info(u"Renaming customize.py to %s", old_customize_file)
    os.rename(customize_file, old_customize_file)

    try:
        # If there are new commandline parameters, append them to the old commandline
        # list for each param type.
        message_destinations = merge_codegen_params(codegen_params.get("message_destinations", []), args.messagedestination)
        functions            = merge_codegen_params(codegen_params.get("functions", []), args.function)
        rules                = merge_codegen_params(codegen_params.get("actions", []), args.rule)
        workflows            = merge_codegen_params(codegen_params.get("workflows", []), args.workflow)
        incident_fields      = merge_codegen_params(codegen_params.get("incident_fields", []), args.field)
        datatables           = merge_codegen_params(codegen_params.get("datatables", []), args.datatable)
        automatic_tasks      = merge_codegen_params(codegen_params.get("automatic_tasks", []), args.task)
        scripts              = merge_codegen_params(codegen_params.get("scripts", []), args.script)
        artifact_types       = merge_codegen_params(codegen_params.get("incident_artifact_types", []), args.artifacttype)

        # Print the codegen --reload command with all arguments.
        print_codegen_reload_commandline(args.reload, args.exportfile,
                                         message_destinations,
                                         functions,
                                         workflows,
                                         rules,
                                         incident_fields,
                                         datatables,
                                         automatic_tasks,
                                         scripts,
                                         artifact_types)

        # Call codegen to recreate package with the new parameter list.
        codegen_package(client,
                    args.exportfile,
                    args.reload,
                    message_destinations,
                    functions,
                    workflows,
                    rules,
                    incident_fields,
                    datatables,
                    automatic_tasks,
                    scripts,
                    artifact_types,
                    output_base)
    except Exception as e:
        LOG.error(u"Error running codegen --reload %s", e)
    finally:
        # If no customize.py was created an error occurred somewhere in codegen.
        # Rename the saved off version back to customize.py
        if not os.path.isfile(customize_file):
            LOG.info(u"Renaming %s back to %s", old_customize_file, customize_file)
            os.rename(old_customize_file, customize_file)

def create_command(command, params, quotes):
    """Create commandline substring for codegen --reload commandline """
    result_command = command
    if len(params) > 0:
        for item in params:
            if quotes:
                result_command = result_command + u' "{}"'.format(item)
            else:
                result_command = result_command + u" {}".format(item)
    else:
        result_command = u""
    return result_command

def print_codegen_reload_commandline(package, export_file, message_destinations, functions, workflows,
                                     rules, incident_fields, datatables, tasks, scripts, artifact_types):
    """Print the resilient-circuits codegen --reload commandline for a given package"""

    # Build the commandline string
    commandline = u"resilient-circuits codegen --reload {}".format(package)
    if export_file:
        commandline = commandline + u" --export {}".format(export_file)
    commandline = commandline + create_command(u" --messagedestination", message_destinations, False)
    commandline = commandline + create_command(u" --rule", rules, True)
    commandline = commandline + create_command(u" --workflow", workflows, False)
    commandline = commandline + create_command(u" --function", functions, False)
    commandline = commandline + create_command(u" --field", incident_fields, False)
    commandline = commandline + create_command(u" --datatable", datatables, False)
    commandline = commandline + create_command(u" --task", tasks, False)
    commandline = commandline + create_command(u" --script", scripts, True)
    commandline = commandline + create_command(u" --artifacttype", artifact_types, True)

    print (commandline)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2018. All Rights Reserved.

"""Action Module server - restartable version of App"""

from __future__ import print_function

import logging
import os
import filelock
import resilient
from circuits import Event, Timer
from watchdog.observers import Observer
from watchdog.events import PatternMatchingEventHandler

from resilient_circuits.app import App, AppArgumentParser
from resilient_circuits.app import get_lock


application = None
LOG = logging.getLogger(__name__)


class reload(Event):
    """Notify components of updates to config"""
    complete = True

    def __init__(self, opts):
        super(reload, self).__init__(opts=opts)
        self.opts = opts


class ConfigFileUpdateHandler(PatternMatchingEventHandler):
    """ Restarts application when config file is modified """

    def __init__(self, app):
        super(ConfigFileUpdateHandler, self).__init__()
        self.app = app
        self.max_reload_time = 30

    @classmethod
    def set_patterns(cls, config_file):
        cls.patterns = ["*" + os.path.basename(config_file), ]

    def on_modified(self, event):
        """ reload data from config file and restart components """
        if self.app.reloading:
            LOG.warning("Configuration file change ignored because reload already in progress")
            return

        LOG.info("Configuration file has changed! Notify components to reload")
        self.app.reloading = True
        opts = AppArgumentParser().parse_args()
        reload_event = reload(opts=opts)
        self.app.reload_timer = Timer(self.max_reload_time, Event.create("reload_timeout"))
        self.app.fire(reload_event)
        self.app.reload_timer.register(self.app)


# Main component for our application
class AppRestartable(App):
    """Our main app component, which sets up the Resilient services and other components"""

    def __init__(self, *args, **kwargs):
        super(AppRestartable, self).__init__(*args, **kwargs)
        self.reloading = False
        self.reload_timer = None
        self.observer = None

    def do_initialize_watchdog(self):
        """Initialize the configuration file watchdog"""
        # Monitor the configuration file, using a Watchdog observer daemon.
        LOG.info("Monitoring config file for changes.")
        ConfigFileUpdateHandler.set_patterns(self.config_file)
        event_handler = ConfigFileUpdateHandler(self)
        self.observer = Observer()
        config_dir = os.path.dirname(self.config_file)
        if not config_dir:
            config_dir = os.getcwd()
        self.observer.schedule(event_handler, path=config_dir, recursive=False)
        self.observer.daemon = True
        self.observer.start()

    def started(self, component):
        LOG.info("App Started %s", str(component))
        self.do_initialize_watchdog()

    def stopped(self, component):
        """Stopped Event Handler"""
        LOG.info("App Stopped")
        self._stop_observer()

    def reload_complete(self, event, *args, **kwargs):
        """ All components done handling reload event """
        if event.parent.success:
            LOG.info("Reload completed successfully!")
        else:
            LOG.error("Reloading failed to complete successfully!")
        if self.reload_timer:
            self.reload_timer.unregister()
            self.reload_timer = None
        self.reloading = False

    def reload_timeout(self, event):
        """Reload timed out, assume it failed"""
        LOG.error("Reload Timed Out, Assuming Failure!")
        self.reloading = False

    def _stop_observer(self):
        """ stop monitoring config file for changes """
        if self.observer:
            LOG.info("Stopping config file monitoring")
            self.observer.unschedule_all()
            self.observer.stop()
            self.observer = None


def run(*args, **kwargs):
    """Main app"""

    # define lock
    # this prevents multiple, identical circuits from running at the same time
    lock = get_lock()

    # The main app component initializes the Resilient services
    global application
    try:
        # attempt to lock file, wait 1 second for lock
        with lock.acquire(timeout=1):
            assert lock.is_locked
            application = AppRestartable(*args, **kwargs)
            application.run()

    except filelock.Timeout:
        # file is probably already locked
        errmsg = ("Failed to acquire lock on {0} - you may have "
                  "another instance of Resilient Circuits running")
        print(errmsg.format(os.path.abspath(lock.lock_file)))
    except ValueError:
        LOG.exception("ValueError Raised. Application not running.")
    except OSError as exc:
        # Some other problem accessing the lockfile
        print("Unable to lock {0}: {1}".format(os.path.abspath(lock.lock_file), exc))
    # finally:
    #    LOG.info("App finished.")


if __name__ == "__main__":
    run()



import unittest
from resilient_circuits.util.resilient_ext import ext_command_handler
from resilient_circuits.util.ext import ExtException


class TestExtCmdHandler(unittest.TestCase):

    def test_unsupported_cmd(self):

        with self.assertRaises(ExtException) as cm:
            ext_command_handler("mock_cmd", {"path_to_package": "/mock/path"})

        the_exception = cm.exception
        self.assertIn("Unsupported command", the_exception.message)



#!/usr/bin/env python
# -*- coding: utf-8 -*-

from setuptools import setup, find_packages
import glob
import ntpath

def get_module_name(module_path):
    """
    Return the module name of the module path
    """
    return ntpath.split(module_path)[1].split(".")[0]

def snake_to_camel(word):
    """
    Convert a word from snake_case to CamelCase
    """
    return ''.join(x.capitalize() or '_' for x in word.split('_'))

setup(
    name='fn_mock_integration',
    version='1.0.0',
    license='MIT License',
    author='Example Author Name',
    author_email='aryastark@example.com',
    url='www.example.com',
    description="An example description for the purpose of unit tests",
    long_description="""Lorem ipsum dolor sit amet, consectetur adipiscing elit, sed do 
    eiusmod tempor incididunt ut labore et dolore magna aliqua. Mauris cursus mattis 
    molestie a iaculis. Cursus risus at ultrices mi tempus imperdiet nulla 
    malesuada pellentesque. Curabitur vitae nunc sed velit. Quis imperdiet massa 
    tincidunt nunc pulvinar sapien et. Morbi blandit cursus risus at ultrices mi 
    tempus imperdiet. Risus ultricies tristique nulla aliquet enim tortor at auctor urna""",
    install_requires=[
        'resilient_circuits>=30.0.0'
    ],
    packages=find_packages(),
    include_package_data=True,
    platforms='any',
    classifiers=[
        'Programming Language :: Python',
    ],
    entry_points={
        "resilient.circuits.components": [
            # When setup.py is executed, loop through the .py files in the components directory and create the entry points.
            "{}FunctionComponent = fn_mock_integration.components.{}:FunctionComponent".format(snake_to_camel(get_module_name(filename)), get_module_name(filename)) for filename in glob.glob("./fn_mock_integration/components/[a-zA-Z]*.py")
        ],
        "resilient.circuits.configsection": ["gen_config = fn_mock_integration.util.config:config_section_data"],
        "resilient.circuits.customize": ["customize = fn_mock_integration.util.customize:customization_data"],
        "resilient.circuits.selftest": ["selftest = fn_mock_integration.util.selftest:selftest_function"]
    }
)


import unittest
import os
import sys
from resilient_circuits.util.ext.ExtCreate import PREFIX_EXTENSION_ZIP
from resilient_circuits.util.ext import ExtConvert, ExtException

# Import mock_data (need to add path to support relative imports in PY3)
sys.path.insert(0, os.path.dirname(os.path.dirname(os.path.abspath(__file__))))
from ext_tests.mock_data import mock_data

path_this_dir = os.path.dirname(os.path.realpath(__file__))
path_temp_test_dir = os.path.join(path_this_dir, "test_temp")
path_mock_built_distributions = os.path.join(path_this_dir, "mock_data", "built_distributions")

path_mock_bd_true_tar = os.path.join(path_mock_built_distributions, "true_tar_fn_mock_integration-1.0.0.tar.gz")
path_mock_bd_true_zip = os.path.join(path_mock_built_distributions, "true_zip_fn_mock_integration-1.0.0.zip")
path_mock_bd_zipped_tar = os.path.join(path_mock_built_distributions, "zipped_tar_fn_mock_integration-1.0.0.zip")
path_mock_bd_zipped_tar_in_sub_folder = os.path.join(path_mock_built_distributions, "zipped_tar_in_sub_folder.zip")
path_mock_bd_corrupt_tar = os.path.join(path_mock_built_distributions, "mock_corrupt_tar.tar.gz")
path_mock_bd_zipped_tar_corrupt = os.path.join(path_mock_built_distributions, "zipped_tar_corrupt.zip")
path_mock_setup_py = os.path.join(path_this_dir, "mock_data", "fn_mock_integration", "setup.py")

# Generate name and path for .zip that is expected to be created when convert completes
expected_extension_zip_name = "{0}{1}-{2}.zip".format(PREFIX_EXTENSION_ZIP, mock_data.MOCK_INTEGRATION_NAME, mock_data.MOCK_INTEGRATION_VERSION)
expected_path_the_extension_zip = os.path.join(path_mock_built_distributions, expected_extension_zip_name)


class ExtConvertClassTestConvertExtension(unittest.TestCase):

    maxDiff = None

    def setUp(self):
        # assertRaisesRegexp renamed to assertRaisesRegex in PY3.2
        if sys.version_info < (3, 2):
            self.assertRaisesRegex = self.assertRaisesRegexp

        self.ext_creator = ExtConvert("ext:convert")

    def tearDown(self):
        # Remove the extension.zip if its created
        if os.path.exists(expected_path_the_extension_zip):
            os.remove(expected_path_the_extension_zip)

    def test_convert_true_tar(self):
        # Convert the built distribution
        path_the_extension_zip = self.ext_creator.convert_to_extension(path_built_distribution=path_mock_bd_true_tar)

        self.assertTrue(os.path.exists(path_the_extension_zip))
        self.assertEqual(path_the_extension_zip, expected_path_the_extension_zip)

    def test_convert_true_zip(self):
        # Convert the built distribution
        path_the_extension_zip = self.ext_creator.convert_to_extension(path_built_distribution=path_mock_bd_true_zip)

        self.assertTrue(os.path.exists(path_the_extension_zip))
        self.assertEqual(path_the_extension_zip, expected_path_the_extension_zip)

    def test_convert_zipped_tar(self):
        # Convert the built distribution
        path_the_extension_zip = self.ext_creator.convert_to_extension(path_built_distribution=path_mock_bd_zipped_tar)

        self.assertTrue(os.path.exists(path_the_extension_zip))
        self.assertEqual(path_the_extension_zip, expected_path_the_extension_zip)

    def test_convert_zipped_tar_in_sub_folder(self):
        # Convert the built distribution
        path_the_extension_zip = self.ext_creator.convert_to_extension(path_built_distribution=path_mock_bd_zipped_tar_in_sub_folder)

        self.assertTrue(os.path.exists(path_the_extension_zip))
        self.assertEqual(path_the_extension_zip, expected_path_the_extension_zip)

    def test_convert_corrupt_tar(self):
        # Convert the built distribution
        with self.assertRaisesRegex(ExtException, "Invalid built distribution"):
            self.ext_creator.convert_to_extension(path_built_distribution=path_mock_bd_corrupt_tar)

    def test_convert_zipped_tar_corrupt(self):
        # Convert the built distribution
        with self.assertRaisesRegex(ExtException, "Could not extract required files from given Built Distribution"):
            self.ext_creator.convert_to_extension(path_built_distribution=path_mock_bd_zipped_tar_corrupt)

    def test_convert_directory(self):
        # Try convert by passing a path to a directory
        with self.assertRaisesRegex(ExtException, "You must specify a Built Distribution. Not a Directory"):
            self.ext_creator.convert_to_extension(path_built_distribution=path_mock_built_distributions)

    def test_convert_file(self):
        # Try convert by passing a path to a file
        with self.assertRaisesRegex(ExtException, "Invalid Built Distribution provided"):
            self.ext_creator.convert_to_extension(path_built_distribution=path_mock_setup_py)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# (c) Copyright IBM Corp. 2010, 2019. All Rights Reserved.
# pylint: disable=line-too-long

""" Python Module that exposes the ExtCreate class """

import logging
import os
import io
import json
import re
import shutil
import sys
import base64
import importlib
import struct
import pkg_resources
from jinja2 import Environment, PackageLoader
from resilient_circuits.util.resilient_customize import ImportDefinition
from resilient_circuits.util.ext.Ext import Ext
from resilient_circuits.util.ext.ExtException import ExtException

# Handle import of ConfigParser + importlib.reload() for PY 2 + 3
if sys.version_info.major < 3:
    import ConfigParser as configparser
else:
    import configparser

    # reload(package) in PY2.7, importlib.reload(package) in PY3.6
    reload = importlib.reload

# Get the same logger object that is used in resilient_circuits_cmd.py
LOG = logging.getLogger("resilient_circuits_cmd_logger")

# Constants
BASE_NAME_BUILD = "build"
BASE_NAME_EXTENSION_JSON = "extension.json"
BASE_NAME_EXPORT_RES = "export.res"
BASE_NAME_EXECUTABLES = "executables"
BASE_NAME_EXECUTABLE_JSON = "executable.json"
BASE_NAME_EXECUTABLE_DOCKERFILE = "Dockerfile"

PREFIX_EXECUTABLE_ZIP = "exe-"
PREFIX_EXTENSION_ZIP = "ext-"

JINJA_TEMPLATE_DOCKERFILE = "docker_file_template.jinja2"

PATH_DEFAULT_ICON_EXTENSION_LOGO = pkg_resources.resource_filename("resilient_circuits", "data/ext/icons/extension_logo.png")
PATH_DEFAULT_ICON_COMPANY_LOGO = pkg_resources.resource_filename("resilient_circuits", "data/ext/icons/company_logo.png")

# Default incident_type to remove from customize.py (if found)
DEFAULT_INCIDENT_TYPE_UUID = u'bfeec2d4-3770-11e8-ad39-4a0004044aa0'


class ExtCreate(Ext):
    """ ExtCreate is a subclass of Ext. It is inherited by
    ExtPackage and ExtConvert. It exposes one method: create_extension() """

    # Tuple of the setup.py attributes we use
    supported_setup_py_attribute_names = (
        "author", "name", "version",
        "description", "long_description", "url"
    )

    # Tuple of all Resilient Object Names we support when packaging/converting
    supported_res_obj_names = (
        "actions", "automatic_tasks", "fields",
        "functions", "incident_artifact_types",
        "incident_types", "message_destinations",
        "phases", "roles", "scripts",
        "types", "workflows", "workspaces"
    )

    # Instansiate Jinja2 Environment with path to Jinja2 templates
    jinja_env = Environment(
        loader=PackageLoader("resilient_circuits", "data/ext/templates"),
        trim_blocks=True
    )

    @staticmethod
    def __get_import_definition_from_customize_py__(path_customize_py_file):
        """Return the base64 encoded ImportDefinition in a customize.py file as a Dictionary"""

        # Insert the customize.py parent dir to the start of our Python PATH at runtime so we can import the customize module from within it
        path_to_util_dir = os.path.dirname(path_customize_py_file)
        sys.path.insert(0, path_to_util_dir)

        # Import the customize module
        customize_py = importlib.import_module("customize")

        # Reload the module so we get the latest one
        # If we do not reload, can get stale results if
        # this method is called more then once
        reload(customize_py)

        # Call customization_data() to get all ImportDefinitions that are "yielded"
        customize_py_import_definitions_generator = customize_py.customization_data()
        customize_py_import_definitions = []

        # customization_data() returns a Generator object with all yield statements, so we loop them
        for definition in customize_py_import_definitions_generator:
            if isinstance(definition, ImportDefinition):
                customize_py_import_definitions.append(json.loads(base64.b64decode(definition.value)))
            else:
                LOG.warning("WARNING: Unsupported data found in customize.py file. Expected an ImportDefinition. Got: '%s'", definition)

        # If no ImportDefinition found
        if not customize_py_import_definitions:
            raise ExtException("No ImportDefinition found in the customize.py file")

        # If more than 1 found
        elif len(customize_py_import_definitions) > 1:
            raise ExtException("Multiple ImportDefinitions found in the customize.py file. There must only be 1 ImportDefinition defined")

        # Get the import defintion as dict
        customize_py_import_definition = customize_py_import_definitions[0]

        # Get reference to incident_types if there are any
        incident_types = customize_py_import_definition.get("incident_types", [])

        if incident_types:

            incident_type_to_remove = None

            # Loop through and remove this custom one (that is originally added using codegen)
            for incident_type in incident_types:
                if incident_type.get("uuid") == DEFAULT_INCIDENT_TYPE_UUID:
                    incident_type_to_remove = incident_type
                    break

            if incident_type_to_remove:
                incident_types.remove(incident_type_to_remove)

        # Remove the path from PYTHONPATH
        sys.path.remove(path_to_util_dir)

        return customize_py_import_definition

    @staticmethod
    def __get_configs_from_config_py__(path_config_py_file):
        """Returns a tuple (config_str, config_list). If no configs found, return ("", []).
        Raises Exception if it fails to parse configs
        - config_str: is the full string found in the config.py file
        - config_list: is a list of dict objects that contain each un-commented config
            - Each dict object has the attributes: name, placeholder, env_name, section_name
        """

        config_str, config_list = "", []

        # Insert the customize.py parent dir to the start of our Python PATH at runtime so we can import the customize module from within it
        path_to_util_dir = os.path.dirname(path_config_py_file)
        sys.path.insert(0, path_to_util_dir)

        try:
            # Import the config module
            config_py = importlib.import_module("config")

            # Reload the module so we get the latest one
            # If we do not reload, can get stale results if
            # this method is called more then once
            reload(config_py)

            # Call config_section_data() to get the string containing the configs
            config_str = config_py.config_section_data()

            # Instansiate a new configparser
            config_parser = configparser.ConfigParser()

            # Read and parse the configs from the config_str
            if sys.version_info < (3, 2):
                # config_parser.readfp() was deprecated and replaced with read_file in PY3.2
                config_parser.readfp(io.StringIO(config_str))

            else:
                config_parser.read_file(io.StringIO(config_str))

            # Get the configs from each section
            for section_name in config_parser.sections():

                parsed_configs = config_parser.items(section_name)

                for config in parsed_configs:
                    config_list.append({
                        "name": config[0],
                        "placeholder": config[1],
                        "env_name": "{0}_{1}".format(section_name.upper(), config[0].upper()),
                        "section_name": section_name
                    })

        except Exception as err:
            raise ExtException("Failed to parse configs from config.py file\nThe config.py file may be corrupt. Visit the App Exchange to contact the developer\nReason: {0}".format(err))

        finally:
            # Remove the path from PYTHONPATH
            sys.path.remove(path_to_util_dir)

        return (config_str, config_list)

    @staticmethod
    def __is_setup_attribute__(line):
        """Use RegEx to check if the given file line starts with (for example) 'long_description='.
        Will also handle if the attribute has been commented out: '# long_description='.
        Returns True if something like 'long_description=' is at the start of the line, else False"""

        any_attribute_regex = re.compile(r'^#?\s*[a-z_]+=')

        if re.match(pattern=any_attribute_regex, string=line) is not None:
            return True

        return False

    @staticmethod
    def __parse_setup_attribute__(path_to_setup_py, setup_py_lines, attribute_name):
        """Returns the attribute value from setup.py given the attribute name"""

        # Generate the regex to find the attribute line
        the_attribute_regex = re.compile("^{0}=".format(attribute_name))

        # Store the contents of the attribute
        the_attribute_found, the_attribute_value = False, []

        # Loop the setup_py lines
        for i in range(len(setup_py_lines)):
            the_attribute_line = setup_py_lines[i]

            # Check if this line contains the attribute_name
            if re.match(pattern=the_attribute_regex, string=the_attribute_line):

                # If found, flip flag + append the line to the_attribute_value
                the_attribute_found = True
                the_attribute_value.append(re.split(pattern=the_attribute_regex, string=the_attribute_line)[1])

                # Check preceding lines to see if this attribute value is multiline string
                for preceding_line in setup_py_lines[i + 1:]:
                    if ExtCreate.__is_setup_attribute__(preceding_line):
                        break

                    # Append the line if it is not a new attribute
                    the_attribute_value.append(preceding_line)

                break

        # If we could not find an attribute with attribute_name, log a warning
        if not the_attribute_found:
            LOG.warning("WARNING: '%s' is not a defined attribute name in the provided setup.py file: %s", attribute_name, path_to_setup_py)

        # Create single string and trim (" , ' )
        the_attribute_value = " ".join(the_attribute_value)
        the_attribute_value = the_attribute_value.strip("\",'.")

        return the_attribute_value

    @staticmethod
    def __parse_setup_py__(path, attribute_names):
        """Parse the values of the given attribute_names and return a Dictionary attribute_name:attribute_value"""

        # Read the setup.py file into a List
        setup_py_lines = ExtCreate.__read_file__(path)

        # Raise an error if nothing found in the file
        if not setup_py_lines:
            raise ExtException("No content found in provided setup.py file: {0}".format(path))

        setup_regex_pattern = r"setup\("
        setup_defined, index_of_setup, return_dict = False, None, dict()

        for i in range(len(setup_py_lines)):

            if re.match(pattern=setup_regex_pattern, string=setup_py_lines[i]) is not None:
                setup_defined = True
                index_of_setup = i
                break

        # Raise an error if we can't find 'setup()' in the file
        if not setup_defined:
            raise ExtException("Could not find 'setup()' defined in provided setup.py file: {0}".format(path))

        # Get sublist containing lines from 'setup(' to EOF + trim whitespaces
        setup_py_lines = setup_py_lines[index_of_setup:]
        setup_py_lines = [file_line.strip() for file_line in setup_py_lines]

        # Foreach attribute_name, get its value and add to return_dict
        for attribute_name in attribute_names:
            return_dict[attribute_name] = ExtCreate.__parse_setup_attribute__(path, setup_py_lines, attribute_name)

        return return_dict

    @staticmethod
    def __get_icon__(icon_name, path_to_icon, width_accepted, height_accepted, default_path_to_icon):
        """Returns the icon at path_to_icon as a base64 encoded string if it is a valid .png file with the resolution
        width_accepted x height_accepted. If path_to_icon does not exist, default_path_to_icon is returned as a base64
        encoded string"""

        path_icon_to_use = path_to_icon

        # Use default_path_to_icon if path_to_icon does not exist
        if not path_icon_to_use or not os.path.isfile(path_icon_to_use):
            LOG.warning("WARNING: Default Extension Icon will be used\nProvided custom icon path for %s is invalid: %s\nNOTE: %s should be placed in the /icons directory", icon_name, path_icon_to_use, icon_name)
            path_icon_to_use = default_path_to_icon

        else:
            LOG.info("INFO: Using custom %s icon: %s", icon_name, path_icon_to_use)

        # Validate path_icon_to_use and ensure we have READ permissions
        try:
            ExtCreate.__validate_file_paths__(os.R_OK, path_icon_to_use)
        except ExtException as err:
            raise OSError("Could not find valid icon file. Looked at two locations:\n{0}\n{1}\n{2}".format(path_to_icon, default_path_to_icon, err.message))

        # Get the extension of the file. os.path.splitext returns a Tuple with the file extension at position 1 and can be an empty string
        split_path = os.path.splitext(path_icon_to_use)
        file_extension = split_path[1]

        if not file_extension:
            raise ExtException("Provided icon file does not have an extension. Icon file must be .png\nIcon File: {0}".format(path_icon_to_use))

        elif file_extension != ".png":
            raise ExtException("{0} is not a supported icon file type. Icon file must be .png\nIcon File: {1}".format(file_extension, path_icon_to_use))

        # Open the icon_file in Bytes mode to validate its resolution
        with open(path_icon_to_use, mode="rb") as icon_file:
            # According to: https://en.wikipedia.org/wiki/Portable_Network_Graphics#File_format
            # First need to seek 16 bytes:
            #   8 bytes: png signature
            #   4 bytes: IDHR Chunk Length
            #   4 bytes: IDHR Chunk type
            icon_file.seek(16)

            try:
                # Bytes 17-20 = image width. Use struct to unpack big-endian encoded unsigned int
                icon_width = struct.unpack(">I", icon_file.read(4))[0]

                # Bytes 21-24 = image height. Use struct to unpack big-endian encoded unsigned int
                icon_height = struct.unpack(">I", icon_file.read(4))[0]
            except Exception as err:
                raise ExtException("Failed to read icon's resolution. Icon file corrupt. Icon file must be .png\nIcon File: {0}".format(path_icon_to_use))

        # Raise exception if resolution is not accepted
        if icon_width != width_accepted or icon_height != height_accepted:
            raise ExtException("Icon resolution is {0}x{1}. Resolution must be {2}x{3}\nIcon File:{4}".format(icon_width, icon_height, width_accepted, height_accepted, path_icon_to_use))

        # If we get here all validations have passed. Open the file in Bytes mode and encode it as base64 and decode to a utf-8 string
        with open(path_icon_to_use, "rb") as icon_file:
            encoded_icon_string = base64.b64encode(icon_file.read()).decode("utf-8")

        return encoded_icon_string

    @staticmethod
    def __add_tag__(tag_name, list_of_objs):
        """Returns list_of_objs with tag_name added to each object"""

        # Create tag_to_add
        tag_to_add = {
            "tag_handle": tag_name,
            "value": None
        }

        err_msg = "Error adding tag '{0}'. '{1}' (printed above) is not a {2}. Instead a {3} was provided.\nProvided ImportDefinition in the customize.py file may be corrupt"

        # Check list_of_objs is a List
        if not isinstance(list_of_objs, list):
            LOG.error("Error adding tag.\n'list_of_objs': %s", list_of_objs)
            raise ExtException(err_msg.format(tag_name, "list_of_objs", "List", type(list_of_objs)))

        # Loop each object in the List
        for obj in list_of_objs:

            # If its not a dict, error
            if not isinstance(obj, dict):
                LOG.error("Error adding tag.\n'list_of_objs': %s\n'obj': %s", list_of_objs, obj)
                raise ExtException(err_msg.format(tag_name, "obj", "Dictionary", type(obj)))

            # Try get current_tags
            current_tags = obj.get("tags")

            # If None, create new empty List
            if current_tags is None:
                current_tags = []

            # If current_tags is not a list, error
            if not isinstance(current_tags, list):
                LOG.error("Error adding tag.\n'current_tags': %s", current_tags)
                raise ExtException(err_msg.format(tag_name, "current_tags", "List", type(current_tags)))

            # Append our tag_to_add to current_tags
            current_tags.append(tag_to_add)

            # Set the obj's 'tags' value to current_tags
            obj["tags"] = current_tags

        # Return the updated list_of_objs
        return list_of_objs

    @staticmethod
    def __add_tag_to_import_definition__(tag_name, supported_res_obj_names, import_definition):
        """Returns import_definition with tag_name added to each supported_res_object_name found
        in the import_definition"""

        for obj_name in supported_res_obj_names:

            res_object_list = import_definition.get(obj_name)

            if res_object_list:
                res_object_list = ExtCreate.__add_tag__(tag_name, res_object_list)

                # A 'function' object has a list of 'workflows' which also need the tag added to
                if obj_name == "functions":
                    res_functions_list = import_definition.get("functions")

                    for fn in res_functions_list:
                        fn_workflows_list = fn.get("workflows")

                        if fn_workflows_list:
                            fn_workflows_list = ExtCreate.__add_tag__(tag_name, fn_workflows_list)

        return import_definition

    @classmethod
    def create_extension(cls, path_setup_py_file, path_customize_py_file, path_config_py_file, output_dir,
                         path_built_distribution=None, path_extension_logo=None, path_company_logo=None, custom_display_name=None, keep_build_dir=False):
        """ Function that creates The Extension.zip file from the given setup.py, customize.py and config.py files
        and copies it to the output_dir. Returns the path to the Extension.zip
        - path_setup_py_file [String]: abs path to the setup.py file
        - path_customize_py_file [String]: abs path to the customize.py file
        - path_config_py_file [String]: abs path to the config.py file
        - output_dir [String]: abs path to the directory the Extension.zip should be produced
        - path_built_distribution [String]: abs path to a tar.gz Built Distribution
            - if provided uses that .tar.gz
            - else looks for it in the output_dir. E.g: output_dir/package_name.tar.gz
        - path_extension_logo [String]: abs path to the extension_logo.png. Has to be 200x72 and a .png file
            - if not provided uses default icon
        - path_company_logo [String]: abs path to the extension_logo.png. Has to be 100x100 and a .png file
            - if not provided uses default icon
        - custom_display_name [String]: will give the Extension that display name. Default: name from setup.py file
        - keep_build_dir [Boolean]: if True, build/ will not be remove. Default: False
        """

        LOG.info("Creating Extension")

        # Ensure the output_dir exists, we have WRITE access and ensure we can READ setup.py and customize.py
        cls.__validate_directory__(os.W_OK, output_dir)
        cls.__validate_file_paths__(os.R_OK, path_setup_py_file, path_customize_py_file)

        # Parse the setup.py file
        setup_py_attributes = cls.__parse_setup_py__(path_setup_py_file, cls.supported_setup_py_attribute_names)

        # Validate setup.py attributes

        # Validate the name attribute. Raise exception if invalid
        if not cls.__is_valid_package_name__(setup_py_attributes.get("name")):
            raise ExtException("'{0}' is not a valid Extension name. The name attribute must be defined and can only include 'a-z and _'.\nUpdate this value in the setup.py file located at: {1}".format(setup_py_attributes.get("name"), path_setup_py_file))

        # Validate the version attribute. Raise exception if invalid
        if not cls.__is_valid_version_syntax__(setup_py_attributes.get("version")):
            raise ExtException("'{0}' is not a valid Extension version syntax. The version attribute must be defined. Example: version=\"1.0.0\".\nUpdate this value in the setup.py file located at: {1}".format(setup_py_attributes.get("version"), path_setup_py_file))

        # Validate the url supplied in the setup.py file, set to an empty string if not valid
        if not cls.__is_valid_url__(setup_py_attributes.get("url")):
            LOG.warning("WARNING: '%s' is not a valid url. Ignoring.", setup_py_attributes.get("url"))
            setup_py_attributes["url"] = ""

        # Get ImportDefinition from customize.py
        customize_py_import_definition = cls.__get_import_definition_from_customize_py__(path_customize_py_file)

        # Get the tag name
        tag_name = setup_py_attributes.get("name")

        # Add the tag to the import defintion
        customize_py_import_definition = cls.__add_tag_to_import_definition__(tag_name, cls.supported_res_obj_names, customize_py_import_definition)

        # Parse the app.configs from the config.py file
        app_configs = cls.__get_configs_from_config_py__(path_config_py_file)

        # Generate the name for the extension
        extension_name = "{0}-{1}".format(setup_py_attributes.get("name"), setup_py_attributes.get("version"))

        # Generate paths to the directories and files we will use in the build directory
        path_build = os.path.join(output_dir, BASE_NAME_BUILD)
        path_extension_json = os.path.join(path_build, BASE_NAME_EXTENSION_JSON)
        path_export_res = os.path.join(path_build, BASE_NAME_EXPORT_RES)
        path_executables = os.path.join(path_build, BASE_NAME_EXECUTABLES)
        path_executable_zip = os.path.join(path_executables, "{0}{1}".format(PREFIX_EXECUTABLE_ZIP, extension_name))
        path_executable_json = os.path.join(path_executable_zip, BASE_NAME_EXECUTABLE_JSON)
        path_executable_dockerfile = os.path.join(path_executable_zip, BASE_NAME_EXECUTABLE_DOCKERFILE)

        try:
            # If there is an old build directory, remove it first
            if os.path.exists(path_build):
                shutil.rmtree(path_build)

            # Create the directories for the path "/build/executables/exe-<package-name>/"
            os.makedirs(path_executable_zip)

            # If no path_built_distribution is given, use the default: "<output_dir>/<package-name>.tar.gz"
            if not path_built_distribution:
                path_built_distribution = os.path.join(output_dir, "{0}.tar.gz".format(extension_name))

            # Validate the built distribution exists and we have READ access
            cls.__validate_file_paths__(os.R_OK, path_built_distribution)

            # Copy the built distribution to the executable_zip dir and  enforce rename to .tar.gz
            shutil.copy(path_built_distribution, os.path.join(path_executable_zip, "{0}.tar.gz".format(extension_name)))

            # Generate the contents for the executable.json file
            the_executable_json_file_contents = {
                "name": extension_name
            }

            # Write the executable.json file
            cls.__write_file__(path_executable_json, json.dumps(the_executable_json_file_contents, sort_keys=True))

            # NOTE: Dockerfile creation commented out for this release
            '''
            # Load Dockerfile template
            docker_file_template = cls.jinja_env.get_template(JINJA_TEMPLATE_DOCKERFILE)

            # Render Dockerfile template with required variables
            the_dockerfile_contents = docker_file_template.render({
                "extension_name": extension_name,
                "installed_package_name": setup_py_attributes.get("name").replace("_", "-"),
                "app_configs": app_configs[1]
            })

            # Write the Dockerfile
            cls.__write_file__(path_executable_dockerfile, the_dockerfile_contents)
            '''

            # zip the executable_zip dir
            shutil.make_archive(base_name=path_executable_zip, format="zip", root_dir=path_executable_zip)

            # Remove the executable_zip dir
            shutil.rmtree(path_executable_zip)

            # Get the extension_logo (icon) and company_logo (author.icon) as base64 encoded strings
            extension_logo = cls.__get_icon__(
                icon_name=os.path.basename(PATH_DEFAULT_ICON_EXTENSION_LOGO),
                path_to_icon=path_extension_logo,
                width_accepted=200,
                height_accepted=72,
                default_path_to_icon=PATH_DEFAULT_ICON_EXTENSION_LOGO)

            company_logo = cls.__get_icon__(
                icon_name=os.path.basename(PATH_DEFAULT_ICON_COMPANY_LOGO),
                path_to_icon=path_company_logo,
                width_accepted=100,
                height_accepted=100,
                default_path_to_icon=PATH_DEFAULT_ICON_COMPANY_LOGO)

            # Generate the contents for the extension.json file
            the_extension_json_file_contents = {
                "author": {
                    "name": setup_py_attributes.get("author"),
                    "website": setup_py_attributes.get("url"),
                    "icon": {
                        "data": company_logo,
                        "media_type": "image/png"
                    }
                },
                "description": {
                    "content": setup_py_attributes.get("description"),
                    "format": "text"
                },
                "display_name": custom_display_name if custom_display_name is not None else setup_py_attributes.get("name"),
                "icon": {
                    "data": extension_logo,
                    "media_type": "image/png"
                },
                "long_description": {
                    "content": "<div>{0}</div>".format(setup_py_attributes.get("long_description")),
                    "format": "html"
                },
                "minimum_resilient_version": {
                    "major": customize_py_import_definition.get("server_version").get("major"),
                    "minor": customize_py_import_definition.get("server_version").get("minor"),
                    "build_number": customize_py_import_definition.get("server_version").get("build_number"),
                    "version": customize_py_import_definition.get("server_version").get("version")
                },
                "name": setup_py_attributes.get("name"),
                "tag": {
                    "prefix": tag_name,
                    "name": tag_name,
                    "display_name": tag_name,
                    "uuid": cls.__generate_uuid_from_string__(tag_name)
                },
                "uuid": cls.__generate_uuid_from_string__("{0}-{1}".format(setup_py_attributes.get("name"), setup_py_attributes.get("version"))),
                "version": setup_py_attributes.get("version"),
                # TODO: discuss with Sasquatch. Can add the app_config_str here, but will not install
                # TODO: get 'Unrecognized field "app_config_str"' error
                # "app_config_str": app_configs[0]
            }

            # Write the executable.json file
            cls.__write_file__(path_extension_json, json.dumps(the_extension_json_file_contents, sort_keys=True))

            # Write the customize ImportDefinition to the export.res file
            cls.__write_file__(path_export_res, json.dumps(customize_py_import_definition, sort_keys=True))

            # Copy the built distribution to the build dir, enforce rename to .tar.gz
            shutil.copy(path_built_distribution, os.path.join(path_build, "{0}.tar.gz".format(extension_name)))

            # create The Extension Zip by zipping the build directory
            extension_zip_base_path = os.path.join(output_dir, "{0}{1}".format(PREFIX_EXTENSION_ZIP, extension_name))
            extension_zip_name = shutil.make_archive(base_name=extension_zip_base_path, format="zip", root_dir=path_build)
            path_the_extension_zip = os.path.join(extension_zip_base_path, extension_zip_name)

        except ExtException as err:
            raise err

        except Exception as err:
            raise ExtException(err)

        finally:
            # Remove the executable_zip dir. Keep it if user passes --keep-build-dir
            if not keep_build_dir:
                shutil.rmtree(path_build)

        LOG.info("Extension %s created", "{0}{1}".format(PREFIX_EXTENSION_ZIP, extension_name))

        # Return the path to the extension zip
        return path_the_extension_zip



# -*- coding: utf-8 -*-

"""Generate a default configuration-file section for fn_mock_integration"""

from __future__ import print_function


def config_section_data():
    """Produce the default configuration section for app.config,
       when called by `resilient-circuits config [-c|-u]`
    """
    config_data = u"""[fn_mock_integration_sec_one]
mock_config_url=https://api.example.com
mock_config_username=aryastark@example.com
#mock_config_timeout=300
mock_config_password=TheK!ghtKing

[fn_mock_integration_sec_two]
#mock_config_proxy_http=
mock_config_proxy_https=https://example.com
"""
    return config_data
